tests = [("python", "UnitTestChem.py", {}), ("python", "UnitTestChemv2.py", {}),
         ("python", "UnitTestChemAtom.py", {}), ("python", "UnitTestChemBond.py", {}),
         ("python", "UnitTestChemSmarts.py", {}), ("python", "UnitTestFragmentDescriptors.py", {}),
         ("python", "UnitTestGraphDescriptors_2.py", {}), ("python", "UnitTestLipinski.py", {}),
         ("python", "UnitTestMCS.py", {}), ("python", "UnitTestOldBugs.py", {}),
         ("python", "UnitTestSATIS.py", {}), ("python", "UnitTestSmiles.py", {}),
         ("python", "UnitTestSuppliers.py", {}), ("python", "UnitTestSurf.py", {}),
         ("python", "UnitTestMol3D.py", {}), ("python", "UnitTestCatalog.py", {}),
         ("python", "UnitTestDescriptors.py", {}), ("python", "UnitTestInchi.py", {}),
         ("python", "UnitTestFunctionalGroups.py", {}), ("python", "UnitTestCrippen.py", {}),
         ("python", "UnitTestPandasTools.py", {}), ("python", "UnitTestDocTestsChem.py", {}),
         ("python", "UnitTestFeatFinderCLI.py", {}), ("python", "UnitTestQED.py", {}), ("python", "UnitTestSpacialScore.py", {}),
         ("python", "BRICS.py", {}), ("python", "EnumerateStereoisomers.py", {}),
         ("python", "UnitTestSaltRemover.py", {}), ("python", "test_list.py", {
           'dir': 'AtomPairs'
         }), ("python", "test_list.py", {
           'dir': 'ChemUtils'
         }), ("python", "test_list.py", {
           'dir': 'EState'
         }), ("python", "test_list.py", {
           'dir': 'FeatMaps'
         }), ("python", "test_list.py", {
           'dir': 'Fingerprints'
         }), ("python", "test_list.py", {
           'dir': 'Pharm2D'
         }), ("python", "test_list.py", {
           'dir': 'Pharm3D'
         }), ("python", "test_list.py", {
           'dir': 'Subshape'
         }), ("python", "test_list.py", {
           'dir': 'Suppliers'
         }), ("python", "test_list.py", {
           'dir': 'Scaffolds'
         }), ("python", "test_list.py", {
           'dir': 'Draw'
         }), ("python", "test_list.py", {
           'dir': 'Fraggle'
         }), ("python", "test_list.py", {
           'dir': 'SimpleEnum'
         }), ("python", "test_list.py", {
           'dir': 'Features'
         }), ("python", "test_list.py", {
           'dir': 'MolStandardize'
         }), ("python", "UnitTestRegistrationHash.py", {})]

# only attempt the MolKey tests if we have the pre-reqs:
try:
  from rdkit.Chem.MolKey import MolKey
  tests.append(("python", "test_list.py", {'dir': 'MolKey'}))
except ImportError:
  pass

longTests = [
  ("python", "UnitTestArom.py", {}),
  ("python", "UnitTestGraphDescriptors.2.py -l", {}),
  ("python", "UnitTestSurf.py -l", {}),
]

if __name__ == '__main__':
  import sys

  from rdkit import TestRunner
  failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
  sys.exit(len(failed))



# $Id$
#
# Copyright (C) 2003-2006 Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" Unit Test code for Fragment Descriptors

"""
import unittest

from rdkit import Chem
from rdkit.Chem import Fragments


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def _runTest(self, data, fn):
    for smi, tgtVal in data:
      mol = Chem.MolFromSmiles(smi)
      assert mol, "Smiles parsing failed for %s" % (smi)
      count = fn(mol)
      assert count == tgtVal, "bad value (%d != %d) for smiles %s" % (count, tgtVal, smi)

  def test1(self):
    data = [
      ('C=O', 1),
      ('O=CC=O', 2),
      ('O=CC(=O)O', 2),
      ('O=C(O)C(=O)O', 2),
    ]
    self._runTest(data, Fragments.fr_C_O)

  def test2(self):
    data = [
      ('C=O', 1),
      ('O=CC=O', 2),
      ('O=CC(=O)O', 1),
      ('O=C(O)C(=O)O', 0),
    ]
    self._runTest(data, Fragments.fr_C_O_noCOO)

  def test3(self):
    data = [
      ('[O-][N+](C1=C([N+]([O-])=O)C=C([N+]([O-])=O)C=C1)=O', 3),
      ('[O-][N+](C1=CC=CC=C1)=O', 1),
      ('[N+]([O-])(=O)C1OC(=CC=1)/C=N/NC(N)=O', 0),
      ('O=C(O)C(=O)O', 0),
    ]
    self._runTest(data, Fragments.fr_nitro_arom)

  def test4(self):
    data = [
      ('[O-][N+](C1=C([N+]([O-])=O)C=C([N+]([O-])=O)C=C1)=O', 1),
      ('[O-][N+](C1=CC=CC=C1)=O', 1),
      ('[N+]([O-])(=O)C1OC(=CC=1)/C=N/NC(N)=O', 0),
      ('O=C(O)C(=O)O', 0),
    ]
    self._runTest(data, Fragments.fr_nitro_arom_nonortho)

  def test5(self):
    data = [
      ('C1(N=[N+]=[N-])=C(C(=C(C(=C1F)F)F)F)F', 1),
    ]
    self._runTest(data, Fragments.fr_azide)

  def test6(self):
    data = [
      ('C1C=CNC=C1', 1),
      ('C1=CCNC=C1', 1),
      ('C1=CCN=CC1', 1),
      ('C1=CC=NCC1', 1),
      ('C1C=CC(=CC=1[N+]([O-])=O)C2C(=C(NC(=C2C(OC)=O)C)C)C(OCCNCC3=CC=CC=C3)=O', 1),
      ('O=C(O)C(=O)O', 0),
    ]
    self._runTest(data, Fragments.fr_dihydropyridine)

  def test7(self):
    data = [
      ('OC1=C(C(O)=O)C=CC=C1', 0),
      ('OC1=CC=CC=C1', 1),
      ('OC1=C(O)C=CC=C1', 2),
      ('O=C(O)C1=CC(O)=CC=C1', 1),
      ('OC1=C(O)C(C(O)=O)=CC=C1', 1),
      ('OC1=C(C(N)=O)C=CC=C1', 0),
      ('OC1=C(CO)C=CC=C1', 0),
      ('OC1=C(C(OC)=O)C=CC=C1', 1),
      ('C1=CC=NCC1', 0),
      ('C1C=CC(=CC=1[N+]([O-])=O)C2C(=C(NC(=C2C(OC)=O)C)C)C(OCCNCC3=CC=CC=C3)=O', 0),
      ('O=C(O)C(=O)O', 0),
    ]
    self._runTest(data, Fragments.fr_phenol_noOrthoHbond)

  def test8(self):
    data = [
      ('OC1=C(C(O)=O)C=CC=C1', 0),
      ('CC(C)(O)C', 0),
      ('CC(O)C', 1),
      ('C=C(O)C', 1),  # includes enols
      ('OC1=C(O)C=CC=C1', 0),
      ('OC1=C(CO)C=CC=C1', 1),
      ('C1C=CC(=CC=1[N+]([O-])=O)C2C(=C(NC(=C2C(OC)=O)C)C)C(OCCNCC3=CC=CC=C3)=O', 0)
    ]
    self._runTest(data, Fragments.fr_Al_OH_noTert)

  def test9(self):
    data = [
      ('OC1=C(C(O)=O)C=CC=C1', 0),
      ('C12=CC=CC=C1NCCN=C2', 1),
      ('C1=C(C=C2C(=C1)NC(CN=C2C3=C(C=CC=C3)Cl)=O)[N+]([O-])=O', 1),  # clonazepam
      ('C12=C(C(=NCC(N1C)=O)C3=CC=CC=C3)C=C(C=C2)Cl', 1),  # valium
      ('CCC1=CC2C(=NCC(=O)N(C)C=2S1)C3=CC=CC=C3Cl', 0),  # clotiazepam has a 5-mb ring
      ('CN(C)CCN1C(=O)C2=CC=CC=C2NC3C=C(Cl)C=CC1=3', 0),  #clobenzepam has a third fused ring
      ('C1C=CC(=CC=1[N+]([O-])=O)C2C(=C(NC(=C2C(OC)=O)C)C)C(OCCNCC3=CC=CC=C3)=O', 0)
    ]
    self._runTest(data, Fragments.fr_benzodiazepine)

  def test10(self):
    data = [
      ('OC1=C(C(O)=O)C=CC=C1', 1),
      ('c1ccccc1Oc2ccccc2', 2),  # diphenyl ether has 2 sites
      ('COC1=CC=CC=C1', 1),
      ('CN(C)C2=CC=CC=C2', 1),
      ('O=C(C4=CC=CC=C4)C3=CC=CC=C3', 0),
      ('COC1=CC=C(C)C(OC)=C1C', 0),
      ('CN(C)C1=CC=CC=C1OC', 2),
      ('O=C(C2=CC=CC=C2)CC1=CC=CC=C1', 0),
      ('O=C(NC2=CC=CC=C2)CN(C)C1=CC=CC=C1', 2),
      ('O=C(N(C)C2=CC=CC=C2)CC1=CC=CC=C1', 1),  # methylated amide is included
      ('C12=CC=CC=C1OCCC2', 1),
      ('C12=CC=CC=C1OC=C2', 1),  # benzofuran
      ('C12=CC=CC=C1NC=N2', 2),  # benzimidazole hydroxylated at 5 position
      ('CC1=CC=C(C=CO2)C2=C1', 0)
    ]
    self._runTest(data, Fragments.fr_para_hydroxylation)

  def test11(self):
    data = [
      ('C1C(C=C2[C@](C1)([C@]3([C@@](CC2)([C@]4([C@](CC3)([C@@H](CC4)O)C)[H])[H])[H])C)=O',
       1),  # testosterone 6beta hydroxylation includedbut not dienone
      ('CCC=CCCC', 2),
      ('C=CC', 1),
      ('C=CCO', 0)
    ]
    self._runTest(data, Fragments.fr_allylic_oxid)

  def test12(self):
    data = [('c1ccccc1C', 1), ('c1ccccc1CC', 1), ('c1(C)cccc(C)c1CC', 2), ('c1(N)cccc(O)c1CC', 0),
            ('c1cccc(C)c1CC', 2), ('c1ccccc1CN', 0), ('c1ccccc1CCN', 0), ('c1(C)ccccc1CCN', 1)]
    self._runTest(data, Fragments.fr_aryl_methyl)

  def test13(self):
    data = [
      ('CNC1(CCCCC1=O)C2(=CC=CC=C2Cl)', 1),  # ketamine
      ('C1=C(C(=C(C=C1)C)NC(CN(CC)CC)=O)C', 1),  # lidocaine
      ('c1(C)ccccc1CCN', 0)
    ]
    self._runTest(data, Fragments.fr_Ndealkylation1)

  def test14(self):
    data = [
      ('C12(=C(N=CC=C1[C@@H](O)C3(N4CC(C(C3)CC4)([H])C=C)[H])C=CC(=C2)OC)', 0),  # quinine
      ('N12CCC(CC2)CC1', 0),  # bridged N
      ('N1(CCC)CCCCC1', 1),
      ('CCC1(CCCCN(C)C1)C2=CC=CC(O)=C2', 1),  # meptazinol
      ('C1(=C2C3=C(C=C1)C[C@@H]4[C@]5([C@@]3([C@H]([C@H](CC5)O)O2)CCN4CC6CCC6)O)O',
       1),  # nalbuphine
      ('CC1N=C2CCCCN2C(=O)C=1CCN3CCC(CC3)C4=NOC5C=C(F)C=CC4=5', 1),  # risperidone
      ('N1CCOCC1', 0),  # morpholino group
      ('n1ccccc1', 0)
    ]
    self._runTest(data, Fragments.fr_Ndealkylation2)

  def test15(self):
    data = [
      ('C(COC(NC(C)C)=O)(COC(N)=O)(CCC)C', 1),  # carisoprodol
      ('CN(CC3=CSC(C(C)C)=N3)C(N[C@@H]([C@H](C)C)C(N[C@@H](CC4=CC=CC=C4)C[C@H](O)[C@H](CC2=CC=CC=C2)NC(OCC1=CN=CS1)=O)=O)=O',
       1),  # ritonavir
      ('c1(C)ccccc1CCN', 0)
    ]
    self._runTest(data, Fragments.fr_alkyl_carbamate)

  def test16(self):
    data = [
      (r'C1(CCC(O1)=O)(C)CC/C=C\CC', 1),
      ('CN(CC3=CSC(C(C)C)=N3)C(N[C@@H]([C@H](C)C)C(N[C@@H](CC4=CC=CC=C4)C[C@H](O)[C@H](CC2=CC=CC=C2)NC(OCC1=CN=CS1)=O)=O)=O',
       0),  # ritonavir
      ('c1(C)ccccc1CCN', 0)
    ]
    self._runTest(data, Fragments.fr_lactone)

  def test17(self):
    data = [
      ('O=C(C=CC1=CC=CC=C1)C=CC2=CC=CC=C2', 0),  # a,b-unsat. dienone
      ('c1ccccc1-C(=O)-c2ccccc2', 0),
      ('c1ccccc1-C(=O)-CCO', 1),
      ('CC(=O)NCCC', 0)
    ]
    self._runTest(data, Fragments.fr_ketone_Topliss)

  def test18(self):
    data = [
      ('C1=CC(=CC=C1C(N[C@@H](CCC(O)=O)C(O)=O)=O)N(CC2=NC3C(=NC(=NC=3N=C2)N)N)C',
       2),  # methotrexate
      ('S(NC1=NC=CC=N1)(=O)(=O)C2=CC=C(C=C2)N', 1),  # sulfadiazine
      ('S(NC1=NC=CC=N1)(=O)(=O)C2=CC=C(C=C2)', 0),
      ('c1ccccc1-C(=O)-CCO', 0),
      ('NNC(=O)C1C2CCCCC=2N=C3C=CC=CC=13', 1),
      ('NC(=N)C1C2CCCCC=2N=C3C=CC=CC=13', 1),
      ('NNC1C2CCCCC=2N=C3C=CC=CC=13', 1),
      ('NC1C2CCCCC=2N=C3C=CC=CC=13', 1)  # tacrine
    ]
    self._runTest(data, Fragments.fr_ArN)

  def test19(self):
    data = [
      ('CC(C)(C)NCC(O)C1=CC(O)=CC(O)=C1', 1),  # terbulatine
      (r'OCCN1CCN(CC/C=C2\C3=CC=CC=C3SC4C=CC(Cl)=CC2=4)CC1', 1),  # clopenthixol
      ('c1ccccc1-C(=O)-CCO', 0),
      ('CC(=O)NCCC', 0)
    ]
    self._runTest(data, Fragments.fr_HOCCN)

  def test20(self):
    data = [
      ('c1ccccc1OC', 1),
      ('c1ccccc1Oc2ccccc2', 0),
      ('c1ccccc1OCC', 0),
    ]
    self._runTest(data, Fragments.fr_methoxy)

  def test21(self):
    data = [
      (r'C/C(C(C)=O)=N\O', 1),
      ('C(=N/OC(C(=O)O)(C)C)(/C1=CS[N+](=N1)C)C(N[C@@H]2C(N([C@@H]2C)S(O)(=O)=O)=O)=O',
       1),  # aztreonam
      ('c1ccccc1OCC', 0),
    ]
    self._runTest(data, Fragments.fr_oxime)

  def test22(self):
    data = [
      ('CCCF', 1),
      ('c1ccccc1C(C)(C)Br', 1),
      ('c1ccccc1CC(C)(C)Cl', 1),
    ]
    self._runTest(data, Fragments.fr_alkyl_halide)


if __name__ == '__main__':
  unittest.main()



tests = [
  ("python", "UnitTestBuildComposite.py", {}),
  ("python", "UnitTestScreenComposite.py", {}),
  ("python", "UnitTestAnalyzeComposite.py", {}),
]

for d in [
    'Cluster', 'Composite', 'Data', 'DecTree', 'Descriptors', 'InfoTheory', 'KNN', 'ModelPackage',
    'NaiveBayes', 'Neural', 'SLT', 'Scoring'
]:
  tests.append(('python', 'test_list.py', {'dir': d}))

longTests = []

if __name__ == '__main__':
  import sys

  from rdkit import TestRunner
  failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
  sys.exit(len(failed))



#
#  Copyright (C) 2000  greg Landrum
#
""" unit tests for the Neural network trainer implementation

   this basically works out **all** of the network code

"""

import random
import unittest
from io import StringIO

from rdkit.ML.Neural import Network, Trainers
from rdkit.ML.Neural.CrossValidate import CrossValidate, CrossValidationDriver
from rdkit.TestRunner import redirect_stdout


class TrainerTestCase(unittest.TestCase):

  def setUp(self):
    random.seed(23)
    self.trainTol = 0.3
    self.orExamples = [[0, 0, 1, 0.1], [0, 1, 1, .9], [1, 0, 1, .9], [1, 1, 1, .9]]
    self.andExamples = [[0, 0, 1, 0.1], [0, 1, 1, .1], [1, 0, 1, .1], [1, 1, 1, .9]]
    self.xorExamples = [[0, 0, 1, .1], [0, 1, 1, .9], [1, 0, 1, .9], [1, 1, 1, .1]]
    self.linExamples = [[.1, .1], [.2, .2], [.3, .3], [.4, .4], [.8, .8]]

  def _trainExamples(self, ex, arch=[3, 1], useAvgErr=False):
    net = Network.Network(arch)
    t = Trainers.BackProp()
    t.TrainOnLine(ex, net, errTol=self.trainTol, useAvgErr=useAvgErr, silent=True)
    errs = [abs(x[-1] - net.ClassifyExample(x)) for x in ex]
    return net, errs

  def testBackpropOr(self):
    # " testing backprop training on or "
    _, errs = self._trainExamples(self.orExamples)
    assert max(errs) < self.trainTol, 'net did not converge properly on or'

  def testBackpropAnd(self):
    # " testing backprop training on and "
    _, errs = self._trainExamples(self.andExamples)
    assert max(errs) < self.trainTol, 'net did not converge properly on and'

  def testBackpropLin(self):
    # " testing backprop training on a linear function "
    _, errs = self._trainExamples(self.linExamples, arch=[1, 2, 1])
    assert max(errs) < self.trainTol, 'net did not converge properly on linear fit'

    _, errs = self._trainExamples(self.linExamples, arch=[1, 2, 1], useAvgErr=True)
    assert max(errs) < 0.4, 'net did not converge properly on or'

  def test_multipleHiddenLayers(self):
    _, errs = self._trainExamples(self.linExamples, arch=[1, 1, 2, 1])
    assert max(errs) < self.trainTol, 'net did not converge properly on linear fit'

  def test_CrossValidate(self):
    # We just check here that the code works
    net, _ = self._trainExamples(self.orExamples)
    percentage, badExamples = CrossValidate(net, self.orExamples, 0.2)
    self.assertEqual(percentage, 1.0 / 4)
    self.assertEqual(len(badExamples), 1)

    percentage, badExamples = CrossValidate(net, self.orExamples, self.trainTol)
    self.assertEqual(percentage, 0.0)
    self.assertEqual(len(badExamples), 0)

    net, cvError = CrossValidationDriver(self.orExamples + self.orExamples, silent=True)
    self.assertEqual(cvError, 0.5)

    net, cvError = CrossValidationDriver(self.orExamples + self.orExamples, silent=True,
                                         replacementSelection=True)
    self.assertEqual(cvError, 0.0)

    net, cvError = CrossValidationDriver(self.orExamples + self.orExamples, silent=True,
                                         calcTotalError=True)
    self.assertEqual(cvError, 0.25)

    f = StringIO()
    with redirect_stdout(f):
      CrossValidationDriver(self.orExamples + self.orExamples)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2003  greg Landrum and Rational Discovery LLC
#
""" """
import os
import unittest
from io import StringIO

from rdkit.ML.DecTree import ID3, CrossValidate, PruneTree
from rdkit.TestRunner import redirect_stdout


def feq(a, b, tol=1e-4):
  return abs(a - b) <= tol


class TreeTestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1(self):
    # " testing pruning with known results "
    oPts = [
      [0, 0, 1, 0],
      [0, 1, 1, 1],
      [1, 0, 1, 1],
      [1, 1, 0, 0],
      [1, 1, 1, 1],
    ]
    tPts = oPts + [[0, 1, 1, 0], [0, 1, 1, 0]]
    tree = ID3.ID3Boot(oPts, attrs=range(3), nPossibleVals=[2] * 4)
    err, badEx = CrossValidate.CrossValidate(tree, oPts)
    assert err == 0.0, 'bad initial error'
    assert len(badEx) == 0, 'bad initial error'

    # prune with original data, shouldn't do anything
    f = StringIO()
    with redirect_stdout(f):
      PruneTree._verbose = True
      newTree, err = PruneTree.PruneTree(tree, [], oPts)
      PruneTree._verbose = False
    self.assertIn('Pruner', f.getvalue())
    assert newTree == tree, 'improper pruning'

    # prune with train data
    newTree, err = PruneTree.PruneTree(tree, [], tPts)
    assert newTree != tree, 'bad pruning'
    assert feq(err, 0.14286), 'bad error result'

  def test_exampleCode(self):
    f = StringIO()
    with redirect_stdout(f):
      try:
        PruneTree._testRandom()
        self.assertTrue(os.path.isfile('prune.pkl'))
      finally:
        if os.path.isfile('orig.pkl'):
          os.remove('orig.pkl')
        if os.path.isfile('prune.pkl'):
          os.remove('prune.pkl')
    self.assertIn('pruned error', f.getvalue())

    f = StringIO()
    with redirect_stdout(f):
      PruneTree._testSpecific()
    self.assertIn('pruned holdout error', f.getvalue())

    f = StringIO()
    with redirect_stdout(f):
      PruneTree._testChain()
    self.assertIn('pruned holdout error', f.getvalue())


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2020  Greg Landrum and T5 Informatics GmbH
#         All Rights Reserved
#
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import rdMolEnumerator


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def testLinkNodes(self):
    mb = '''one linknode
  Mrv2007 06222005102D          

  0  0  0     0  0            999 V3000
M  V30 BEGIN CTAB
M  V30 COUNTS 6 6 0 0 0
M  V30 BEGIN ATOM
M  V30 1 C 8.25 12.1847 0 0
M  V30 2 C 6.9164 12.9547 0 0
M  V30 3 C 6.9164 14.4947 0 0
M  V30 4 C 9.5836 14.4947 0 0
M  V30 5 C 9.5836 12.9547 0 0
M  V30 6 O 8.25 10.6447 0 0
M  V30 END ATOM
M  V30 BEGIN BOND
M  V30 1 1 1 2
M  V30 2 1 2 3
M  V30 3 1 4 5
M  V30 4 1 1 5
M  V30 5 1 3 4
M  V30 6 1 1 6
M  V30 END BOND
M  V30 LINKNODE 1 4 2 1 2 1 5
M  V30 END CTAB
M  END'''
    m = Chem.MolFromMolBlock(mb)
    ps = rdMolEnumerator.MolEnumeratorParams(rdMolEnumerator.EnumeratorType.LinkNode)
    bndl = rdMolEnumerator.Enumerate(m, ps)
    self.assertEqual(bndl.Size(), 4)

  def testPositionVariation(self):
    mb = '''two position variation bonds
  Mrv2007 06242006032D          

  0  0  0     0  0            999 V3000
M  V30 BEGIN CTAB
M  V30 COUNTS 10 8 0 0 0
M  V30 BEGIN ATOM
M  V30 1 C -1.7083 2.415 0 0
M  V30 2 C -3.042 1.645 0 0
M  V30 3 C -3.042 0.105 0 0
M  V30 4 N -1.7083 -0.665 0 0
M  V30 5 C -0.3747 0.105 0 0
M  V30 6 C -0.3747 1.645 0 0
M  V30 7 * -3.042 0.875 0 0
M  V30 8 F -5.0434 0.875 0 0
M  V30 9 * -1.0415 2.03 0 0
M  V30 10 Cl -1.0415 4.34 0 0
M  V30 END ATOM
M  V30 BEGIN BOND
M  V30 1 1 1 2
M  V30 2 2 2 3
M  V30 3 1 3 4
M  V30 4 2 4 5
M  V30 5 1 5 6
M  V30 6 2 1 6
M  V30 7 1 7 8 ENDPTS=(2 2 3) ATTACH=ANY
M  V30 8 1 9 10 ENDPTS=(2 1 6) ATTACH=ANY
M  V30 END BOND
M  V30 END CTAB
M  END
'''
    m = Chem.MolFromMolBlock(mb)
    ps = rdMolEnumerator.MolEnumeratorParams(rdMolEnumerator.EnumeratorType.PositionVariation)
    bndl = rdMolEnumerator.Enumerate(m, ps)
    self.assertEqual(bndl.Size(), 4)

  def testCombined(self):
    mb = '''
  Mrv2014 12212013392D          

  0  0  0     0  0            999 V3000
M  V30 BEGIN CTAB
M  V30 COUNTS 12 12 0 0 0
M  V30 BEGIN ATOM
M  V30 1 C -6 4.7484 0 0
M  V30 2 C -7.3337 3.9784 0 0
M  V30 3 N -7.3337 2.4383 0 0
M  V30 4 C -6 1.6683 0 0
M  V30 5 C -4.6663 2.4383 0 0
M  V30 6 C -4.6663 3.9784 0 0
M  V30 7 C -5.8773 0.0617 0 0
M  V30 8 C -3.2136 0.2013 0 0
M  V30 9 C -4.5052 -0.6374 0 0
M  V30 10 O -4.4246 -2.1753 0 0
M  V30 11 * -6 4.235 0 0
M  V30 12 C -4.845 6.2355 0 0
M  V30 END ATOM
M  V30 BEGIN BOND
M  V30 1 1 1 2
M  V30 2 2 2 3
M  V30 3 1 3 4
M  V30 4 2 4 5
M  V30 5 1 5 6
M  V30 6 2 1 6
M  V30 7 1 8 9
M  V30 8 1 7 9
M  V30 9 1 5 8
M  V30 10 1 7 4
M  V30 11 1 9 10
M  V30 12 1 11 12 ENDPTS=(3 1 2 6) ATTACH=ANY
M  V30 END BOND
M  V30 LINKNODE 1 3 2 9 7 9 8
M  V30 END CTAB
M  END
'''
    m = Chem.MolFromMolBlock(mb)
    bndl = rdMolEnumerator.Enumerate(m)
    self.assertEqual(bndl.Size(), 9)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""
molvs.validations
~~~~~~~~~~~~~~~~~

This module contains all the built-in :class:`Validations <molvs.validations.Validation>`.

:copyright: Copyright 2016 by Matt Swain.
:license: MIT, see LICENSE file for more details.
"""
from warnings import warn

warn(f'The module {__name__} is deprecated and will be removed in the next release.',
     DeprecationWarning, stacklevel=2)

import logging

from rdkit import Chem

from .errors import StopValidateError
from .fragment import REMOVE_FRAGMENTS


class Validation(object):
  """The base class that all :class:`~molvs.validations.Validation` subclasses must inherit from."""

  def __init__(self, log):
    self.log = logging.LoggerAdapter(log, {'validation': type(self).__name__})

  def __call__(self, mol):
    try:
      self.log.debug('Running %s', type(self).__name__)
      self.run(mol)
    except Exception as e:
      if isinstance(e, StopValidateError):
        raise e
      else:
        self.log.debug('Validation failed: %s', e)

  def run(self, mol):
    """"""
    raise NotImplementedError("Validation subclasses must implement the run method")


class SmartsValidation(Validation):
  """Abstract superclass for :class:`Validations <molvs.validations.Validation>` that log a message if a SMARTS
    pattern matches the molecule.

    Subclasses can override the following attributes:
    """

  #: The logging level of the message.
  level = logging.INFO

  #: The message to log if the SMARTS pattern matches the molecule.
  message = 'Molecule matched %(smarts)s'

  #: Whether the SMARTS pattern should match an entire covalent unit.
  entire_fragment = False

  def __init__(self, log):
    super(SmartsValidation, self).__init__(log)
    self._smarts = Chem.MolFromSmarts(self.smarts)

  @property
  def smarts(self):
    """The SMARTS pattern as a string. Subclasses must implement this."""
    raise NotImplementedError('SmartsValidation subclasses must have a smarts attribute')

  def _check_matches(self, mol):
    if mol.HasSubstructMatch(self._smarts):
      self.log.log(self.level, self.message, {'smarts': self.smarts})

  def _check_matches_fragment(self, mol):
    matches = frozenset(frozenset(match) for match in mol.GetSubstructMatches(self._smarts))
    fragments = frozenset(frozenset(frag) for frag in Chem.GetMolFrags(mol))
    if matches & fragments:
      self.log.log(self.level, self.message, {'smarts': self.smarts})

  def run(self, mol):
    if self.entire_fragment:
      self._check_matches_fragment(mol)
    else:
      self._check_matches(mol)


class IsNoneValidation(Validation):
  """Logs an error if ``None`` is passed to the Validator.

    This can happen if RDKit failed to parse an input format. If the molecule is ``None``, no subsequent validations
    will run.
    """

  def run(self, mol):
    if mol is None:
      self.log.error('Molecule is None')
      raise StopValidateError()


class NoAtomValidation(Validation):
  """Logs an error if the molecule has zero atoms.

    If the molecule has no atoms, no subsequent validations will run.
    """

  def run(self, mol):
    if mol.GetNumAtoms() == 0:
      self.log.error('No atoms are present')
      raise StopValidateError()


class DichloroethaneValidation(SmartsValidation):
  """Logs if 1,2-dichloroethane is present.

    This is provided as an example of how to subclass :class:`~molvs.validations.SmartsValidation` to check for the
    presence of a substructure.
    """
  level = logging.INFO
  smarts = '[Cl]-[#6]-[#6]-[Cl]'
  entire_fragment = True
  message = '1,2-Dichloroethane is present'


class FragmentValidation(Validation):
  """Logs if certain fragments are present.

    Subclass and override the ``fragments`` class attribute to customize the list of
    :class:`FragmentPatterns <molvs.fragment.FragmentPattern>`.
    """

  #: A list of :class:`FragmentPatterns <molvs.fragment.FragmentPattern>` to check for.
  fragments = REMOVE_FRAGMENTS

  def run(self, mol):
    for fp in self.fragments:
      matches = frozenset(frozenset(match) for match in mol.GetSubstructMatches(fp.smarts))
      fragments = frozenset(frozenset(frag) for frag in Chem.GetMolFrags(mol))
      if matches & fragments:
        self.log.info(f'{fp.name} is present', )


class NeutralValidation(Validation):
  """Logs if not an overall neutral system."""

  def run(self, mol):
    charge = Chem.GetFormalCharge(mol)
    if not charge == 0:
      chargestring = '+%s' % charge if charge > 0 else '%s' % charge
      self.log.info(f'Not an overall neutral system ({chargestring})')


class IsotopeValidation(Validation):
  """Logs if molecule contains isotopes."""

  def run(self, mol):
    isotopes = set()
    for atom in mol.GetAtoms():
      isotope = atom.GetIsotope()
      if not isotope == 0:
        isotopes.add(f'{isotope}{atom.GetSymbol()}')
    for isotope in isotopes:
      self.log.info(f'Molecule contains isotope {isotope}')


#: The default list of :class:`Validations <molvs.validations.Validation>` used by :class:`~molvs.validate.Validator`.
VALIDATIONS = (
  IsNoneValidation,
  NoAtomValidation,
  # DichloroethaneValidation,
  FragmentValidation,
  NeutralValidation,
  IsotopeValidation,
)

# - WARN/ERROR: Are all atoms defined/real - no query atoms or invalid elements, r-group things
# - INFO: Contains unknown stereo (Perform stereochemistry perception first?)
# - INFO: Nonstandard tautomer (log SMILES of tautomer parent, or the name of the tautomer transform?)
# - WARN: InChI generation failed
# - WARN: Contains covalent bond to metal (that would be broken by MetalDisconnector)
# - WARN: Contains solvent molecules (in addition other fragment)
# - WARN: More than 99 rings causes problems with SMILES
# - INFO: Cis azo dye is unusual
# - WARN: Adjacent atoms with like charges (i.e. both positive or both negative)
# - INFO: Has more than one radical centre
# - INFO: ethane, methane molecules present
# - INFO: Boron, Sulfur atoms with no explicit bonds
# - INFO: Solvent molecules present (only if also other fragments)
# - INFO: One unknown stereocentre and no defined stereocentres (probably racemate, so info not warn)
# - WARN: More than one undefined stereocentre and no defined stereocentres
# - INFO: One undefined stereocentre and at least one defined stereocentre (epimer or mixture of anomers, so info not warn)
# - WARN: More than one undefined stereocentre and at least one defined stereocentre
# - INFO: Unknown double bond stereochemistry
# - WARN: Ring containing stereobonds?
# - INFO: Not canonical tautomer

# Coordinates?
# Info - Lack of coordinates? Uneven bond lengths?

# Web services (needs to be optional)
# Info - Could not match to ChemSpider ID, PubChem CID
# UniChem from EBI could be useful here, otherwise use each API directly

# Allow definition of MolSchema to set custom validations on e.g.

# People can define a filterer
# This has a series of validations, and the required output - e.g. no error or no warns?



import doctest
import os
import unittest

from rdkit import Chem
from rdkit.Chem.SaltRemover import InputFormat, SaltRemover


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(Chem.SaltRemover, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def test_withSmiles(self):
    remover = SaltRemover(defnData="[Na+]\nCC(=O)O", defnFormat=InputFormat.SMILES)
    self.assertEqual(len(remover.salts), 2)
    mol = Chem.MolFromSmiles('CC(=O)O.[Na+]')
    res = remover.StripMol(mol)
    self.assertEqual(res.GetNumAtoms(), 0)

  def test_withSdfFile(self):
    testFile = os.sep.join(
      [os.path.dirname(os.path.abspath(__file__)), 'test_data', 'witch-salts.sdf'])
    remover = SaltRemover(defnFilename=testFile, defnFormat=InputFormat.MOL)
    self.assertEqual(len(remover.salts), 240)
    m = Chem.MolFromSmiles("Cc1onc(-c2ccccc2)c1C([O-])=NC1C(=O)N2C1SC(C)(C)C2C(=O)O.O.[Na+]")
    tuple = remover.StripMolWithDeleted(m)
    self.assertEqual(Chem.MolToSmiles(tuple.mol),
                     'Cc1onc(-c2ccccc2)c1C([O-])=NC1C(=O)N2C1SC(C)(C)C2C(=O)O.O')
    self.assertEqual(len(tuple.deleted), 1)
    self.assertEqual(Chem.MolToSmiles(tuple.deleted[0]), '[Na+]')

  def test_withSmiFile(self):
    testFile = os.sep.join(
      [os.path.dirname(os.path.abspath(__file__)), 'test_data', 'c6h6-cdk.smi'])
    remover = SaltRemover(defnFilename=testFile, defnFormat=InputFormat.SMILES)
    self.assertEqual(len(remover.salts), 216)

  def test_withDontRemoveEverything(self):
    testFile = os.sep.join(
      [os.path.dirname(os.path.abspath(__file__)), 'test_data', 'witch-salts.sdf'])
    remover = SaltRemover(defnFilename=testFile, defnFormat=InputFormat.MOL)
    m = Chem.MolFromSmiles('Cc1ccccc1')
    mol, deleted = remover.StripMolWithDeleted(m, dontRemoveEverything=True)
    # List should be empty
    self.assertFalse(deleted)
    self.assertEqual(m, mol)

  def test_SmilesVsSmarts(self):
    # SMARTS
    remover = SaltRemover(defnData="[Cl,Br]")
    mol = Chem.MolFromSmiles('CN(Br)Cl.Cl')
    res = remover.StripMol(mol)
    self.assertEqual(res.GetNumAtoms(), 4)
    self.assertEqual(Chem.MolToSmiles(res), 'CN(Cl)Br')
    mol = Chem.MolFromSmiles('CN(C)C.Cl.Br')
    res, deleted = remover.StripMolWithDeleted(mol)
    self.assertEqual(Chem.MolToSmiles(res), 'CN(C)C')
    # Because we read in SMARTS, we should output as well. Otherwise, we will have
    # mismatches
    self.assertListEqual([Chem.MolToSmarts(m) for m in deleted], ['[Cl,Br]'])
    # SMILES
    remover = SaltRemover(defnData="Cl", defnFormat=InputFormat.SMILES)
    mol = Chem.MolFromSmiles('CN(Br)Cl.Cl')
    res = remover.StripMol(mol)
    self.assertEqual(res.GetNumAtoms(), 4)
    self.assertEqual(Chem.MolToSmiles(res), 'CN(Cl)Br')

  def test_github_4550(self):
    m = Chem.MolFromSmiles('Cl.C[N]1=CC=CC=C1', sanitize=False)
    self.assertEqual(m.GetNumAtoms(), 8)
    saltstrip = SaltRemover()
    res = saltstrip.StripMol(m, sanitize=False)
    self.assertEqual(Chem.MolToSmiles(res), 'CN1=CC=CC=C1')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import random
import unittest

import numpy

from rdkit import DataStructs, RDConfig
from rdkit.DataManip.Metric import rdMetricMatrixCalc as rdmmc


def feq(v1, v2, tol2=1e-4):
  return abs(v1 - v2) <= tol2


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test0DistsArray(self):
    exp = numpy.array([1., 1.414213, 1.0], 'd')

    # initialize a double array and check if get back the expected distances
    desc = numpy.zeros((3, 2), 'd')
    desc[1, 0] = 1.0
    desc[2, 0] = 1.0
    desc[2, 1] = 1.0

    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

    # repeat with an float array
    desc = numpy.zeros((3, 2), 'f')
    desc[1, 0] = 1.0
    desc[2, 0] = 1.0
    desc[2, 1] = 1.0

    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

    # finally with an integer array
    desc = numpy.zeros((3, 2), 'i')
    desc[1, 0] = 1
    desc[2, 0] = 1
    desc[2, 1] = 1

    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

  def ctest1DistsListArray(self):
    exp = numpy.array([1., 1.414213, 1.0], 'd')

    desc = [
      numpy.array([0.0, 0.0], 'd'),
      numpy.array([1.0, 0.0], 'd'),
      numpy.array([1.0, 1.0], 'd')
    ]
    dmat = rdmmc.GetEuclideanDistMat(desc)

    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

    # repeat the test with a list of numpy.arrays of floats
    desc = [
      numpy.array([0.0, 0.0], 'f'),
      numpy.array([1.0, 0.0], 'f'),
      numpy.array([1.0, 1.0], 'f')
    ]
    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

    # repeat the test with a list of numpy.arrays of ints
    desc = [numpy.array([0, 0], 'i'), numpy.array([1, 0], 'i'), numpy.array([1, 1], 'i')]
    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

  def test2DistListList(self):
    exp = numpy.array([1., 1.414213, 1.0], 'd')

    desc = [[0.0, 0.0], [1.0, 0.0], [1.0, 1.0]]
    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

    # test with ints
    desc = [[0, 0], [1, 0], [1, 1]]
    dmat = rdmmc.GetEuclideanDistMat(desc)
    for i in range(numpy.shape(dmat)[0]):
      assert feq(dmat[i], exp[i])

  def test3Compare(self):
    n = 30
    m = 5

    dscArr = numpy.zeros((n, m), 'd')
    for i in range(n):
      for j in range(m):
        dscArr[i, j] = random.random()
    dmatArr = rdmmc.GetEuclideanDistMat(dscArr)

    dscLL = []
    for i in range(n):
      row = []
      for j in range(m):
        row.append(dscArr[i, j])
      dscLL.append(row)
    dmatLL = rdmmc.GetEuclideanDistMat(dscLL)

    assert numpy.shape(dmatArr) == numpy.shape(dmatLL)

    for i in range(n * (n - 1) // 2):
      assert feq(dmatArr[i], dmatLL[i])

  def test4ebv(self):

    n = 30
    m = 2048
    dm = 800
    lst = []
    for i in range(n):
      v = DataStructs.ExplicitBitVect(m)
      for j in range(dm):
        v.SetBit(random.randrange(0, m))
      lst.append(v)

    dMat = rdmmc.GetTanimotoDistMat(lst)

    sMat = rdmmc.GetTanimotoSimMat(lst)

    for i in range(n * (n - 1) // 2):
      assert feq(sMat[i] + dMat[i], 1.0)

  def test5sbv(self):

    n = 30
    m = 2048
    dm = 800
    lst = []
    for i in range(n):
      v = DataStructs.SparseBitVect(m)
      for j in range(dm):
        v.SetBit(random.randrange(0, m))
      lst.append(v)

    dMat = rdmmc.GetTanimotoDistMat(lst)

    sMat = rdmmc.GetTanimotoSimMat(lst)

    for i in range(n * (n - 1) // 2):
      assert feq(sMat[i] + dMat[i], 1.0)


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2002-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""
unit testing code for the SD file handling stuff
"""
import os
import tempfile
import unittest

from rdkit import Chem, RDConfig


class TestCase(unittest.TestCase):

  def setUp(self):
    self.fName = os.path.join(RDConfig.RDDataDir, 'NCI', 'first_200.props.sdf')
    with open(self.fName, 'r') as inf:
      inD = inf.read()
    self.nMolecules = inD.count('$$$$')

  def assertMolecule(self, mol, i):
    """ Assert that we have a valid molecule """
    self.assertIsNotNone(mol, 'read %d failed' % i)
    self.assertGreater(mol.GetNumAtoms(), 0, 'no atoms in mol %d' % i)

  def test_SDMolSupplier(self):
    # tests reads using a file name (file contains 200 molecules)
    supp = Chem.SDMolSupplier(self.fName)

    # Can use as an iterator
    for i in range(10):
      mol = next(supp)
      self.assertMolecule(mol, i)

    # Can access directly
    i = 100
    mol = supp[i - 1]
    self.assertMolecule(mol, i)

    # We can access the number of molecules
    self.assertEqual(len(supp), self.nMolecules, 'bad supplier length')

    # We know the number and can still access directly
    i = 12
    mol = supp[i - 1]
    self.assertMolecule(mol, i)

    # Get an exception if we access an invalid number
    with self.assertRaises(IndexError):
      _ = supp[self.nMolecules]  # out of bound read must fail

    # and we can access with negative numbers
    mol1 = supp[len(supp) - 1]
    mol2 = supp[-1]
    self.assertEqual(Chem.MolToSmiles(mol1), Chem.MolToSmiles(mol2))

  def test_SDWriter(self):
    # tests writes using a file name
    supp = Chem.SDMolSupplier(self.fName)
    outName = tempfile.NamedTemporaryFile(suffix='.sdf', delete=False).name
    writer = Chem.SDWriter(outName)
    m1 = next(supp)
    writer.SetProps(m1.GetPropNames())
    for m in supp:
      writer.write(m)
    writer.flush()
    writer.close()

    # The writer does not have an explicit "close()" so need to
    # let the garbage collector kick in to close the file.
    writer = None
    with open(outName, 'r') as inf:
      outD = inf.read()
    # The file should be closed, but if it isn't, and this
    # is Windows, then the unlink() can fail. Wait and try again.
    try:
      os.unlink(outName)
    except Exception:
      import time
      time.sleep(1)
      try:
        os.unlink(outName)
      except Exception:
        pass
    self.assertEqual(self.nMolecules, outD.count('$$$$'), 'bad nMols in output')


#   def _testStreamRoundtrip(self):
#     inD = open(self.fName).read()
#     supp = Chem.SDMolSupplier(self.fName)
#     outName = tempfile.mktemp('.sdf')
#     writer = Chem.SDWriter(outName)
#     _ = next(supp)
#     for m in supp:
#       writer.write(m)
#     writer.flush()
#     writer = None
#     outD = open(outName, 'r').read()
#     try:
#       os.unlink(outName)
#     except Exception:
#       import time
#       time.sleep(1)
#       try:
#         os.unlink(outName)
#       except Exception:
#         pass
#     assert inD.count('$$$$') == outD.count('$$$$'), 'bad nMols in output'
#     io = StringIO(outD)
#     supp = Chem.SDMolSupplier(stream=io)
#     outD2 = supp.Dump()
#     assert outD2.count('$$$$') == len(supp), 'bad nMols in output'
#     assert outD2.count('$$$$') == outD.count('$$$$'), 'bad nMols in output'
#     assert outD2 == outD, 'bad outd'

#   def _testLazyDataRoundtrip(self):
#     inD = open(self.fName).read()
#     supp = Chem.SDMolSupplier(self.fName)
#     outName = tempfile.mktemp('.sdf')
#     writer = Chem.SDWriter(outName)
#     _ = next(supp)
#     for m in supp:
#       writer.write(m)
#     writer.flush()
#     writer = None
#     outD = open(outName, 'r').read()
#     try:
#       os.unlink(outName)
#     except Exception:
#       import time
#       time.sleep(1)
#       try:
#         os.unlink(outName)
#       except Exception:
#         pass
#     assert inD.count('$$$$') == outD.count('$$$$'), 'bad nMols in output'
#     supp = Chem.SDMolSupplier.LazySDMolSupplier(inD=outD)
#     outD2 = supp.Dump()
#     assert outD2.count('$$$$') == len(supp), 'bad nMols in output'
#     assert outD2.count('$$$$') == outD.count('$$$$'), 'bad nMols in output'
#     assert outD2 == outD, 'bad outd'

#   def _testLazyIter(self):
#     " tests lazy reads using the iterator interface "
#     supp = Chem.SDMolSupplier.LazySDMolSupplier(fileN=self.fName)
#
#     nDone = 0
#     for mol in supp:
#       assert mol, 'read %d failed' % nDone
#       assert mol.GetNumAtoms(), 'no atoms in mol %d' % nDone
#       nDone += 1
#     assert nDone == 200, 'bad number of molecules: %d' % (nDone)
#
#     l = len(supp)
#     assert l == 200, 'bad supplier length: %d' % (l)
#
#     i = 12
#     m = supp[i - 1]
#     assert m, 'back index %d failed' % i
#     assert m.GetNumAtoms(), 'no atoms in mol %d' % i
#
#     try:
#       m = supp[201]
#     except IndexError:
#       fail = 1
#     else:
#       fail = 0
#     assert fail, 'out of bound read did not fail'

if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
#  Copyright (C) 2007  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#

import doctest
import unittest

from rdkit import Chem, DataStructs
from rdkit.Chem.Fingerprints.FingerprintMols import FingerprinterDetails
from rdkit.DataStructs import (BitUtils, FingerprintSimilarity, LazySignature,
                               VectCollection)


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(BitUtils, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(LazySignature, optionflags=doctest.ELLIPSIS))
  tests.addTests(doctest.DocTestSuite(VectCollection, optionflags=doctest.ELLIPSIS))
  return tests


class TestCaseAdditional(unittest.TestCase):

  def test_VectCollection(self):
    # We mainly test the use of Reset
    bv1 = DataStructs.ExplicitBitVect(10)
    bv1.SetBitsFromList((1, 3, 5))
    bv2 = DataStructs.ExplicitBitVect(10)
    bv2.SetBitsFromList((6, 8))

    vc = VectCollection.VectCollection()
    self.assertEqual(vc.GetOrVect(), None)
    vc.AddVect(1, bv1)
    vc.AddVect(2, bv2)

    onBits = set([1, 3, 5, 6, 8])
    for i, onOff in enumerate(vc.GetOrVect()):
      self.assertEqual(i in onBits, onOff == 1)
    vc.Reset()
    self.assertEqual(onBits, set(vc.GetOnBits()))

    vc = VectCollection.VectCollection()
    self.assertEqual(vc.GetOrVect(), None)
    vc.AddVect(1, bv1)
    vc.AddVect(2, bv2)
    for i in onBits:
      self.assertEqual(vc[i], 1)

  def test_LazySig(self):
    self.assertRaises(ValueError, LazySignature.LazySig, lambda x: 1, 0)

    # Check cache works
    obj = LazySignature.LazySig(lambda x: x, 10)
    self.assertNotIn(8, obj._cache)
    self.assertEqual(obj[8], 8)
    self.assertIn(8, obj._cache)
    obj._cache[8] = 'cached'
    self.assertEqual(obj[8], 'cached')

  def test__init__(self):
    from rdkit.Chem.Fingerprints import FingerprintMols
    ms = [Chem.MolFromSmiles('CCOC'), Chem.MolFromSmiles('CCO'), Chem.MolFromSmiles('COC')]
    fps = [FingerprintMols.FingerprintMol(x) for x in ms]
    self.assertAlmostEqual(FingerprintSimilarity(fps[0], fps[1]), 0.6, places=2)

    details = FingerprinterDetails()
    fpArgs = details.__dict__
    fps = []
    for i, x in enumerate(ms, 1):
      fpArgs['fpSize'] = 16 * i
      fps.append(FingerprintMols.FingerprintMol(x, **fpArgs))
    self.assertAlmostEqual(FingerprintSimilarity(fps[0], fps[1]), 0.555, places=2)
    self.assertAlmostEqual(FingerprintSimilarity(fps[1], fps[0]), 0.555, places=2)

    fpArgs['fpSize'] = 1024
    fpArgs['tgtDensity'] = 0.8
    fp = FingerprintMols.FingerprintMol(ms[0], **fpArgs)
    self.assertEqual(len(fp), 64)
    fp = DataStructs.FoldToTargetDensity(fp, density=0.1, minLength=2)
    self.assertEqual(len(fp), 4)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#from DataStructs import cDataStructs
import moduleA
import moduleB
import moduleC
from DataStructs import cDataStructs

print("*****************************")
print("Testing self print for classA")
A = moduleA.classA()
A.printA()
print("*****************************")
print()

print("*****************************")
print("Testing self print for classC")
C = moduleC.classC()
C.printC()
print("*****************************")
print()

print("*****************************")
print("Testing crossing C to B")
moduleB.testCrossC(C)
print("*****************************")
print()

cc = moduleB.GetClassC()
cc.printC()

fp = cDataStructs.ExplicitBitVect(20)

print("created fp")

fp = moduleB.GetEBV()

#print("*****************************")
#print("Testing crossing A to B")
#moduleB.testCrossA(A)
#print("*****************************")



#
# Created by Peter Gedeck, January 2017
#

import doctest
import os
import unittest

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.SimpleEnum import Enumerator


def load_tests(loader, tests, ignore):
  """ Add the Doctests from the module """
  tests.addTests(doctest.DocTestSuite(Enumerator, optionflags=doctest.ELLIPSIS))
  return tests


class TestCase(unittest.TestCase):

  def test_EnumerateReactionException(self):
    testFile = os.sep.join(
      [os.path.dirname(os.path.abspath(__file__)), 'test_data', 'boronic1.rxn'])
    rxn = AllChem.ReactionFromRxnFile(testFile)
    rxn.Initialize()
    reacts1 = ['Brc1ccccc1', 'Brc1ncccc1', 'Brc1cnccc1']
    reacts1 = [Chem.MolFromSmiles(x) for x in reacts1]

    self.assertRaises(ValueError, Enumerator.EnumerateReaction, rxn, (reacts1, ))

  def test_UniquifyingEnhancedStereo(self):
    rxn = AllChem.ReactionFromSmarts('[C:1]-[O:2]>>[C:1]-[N:2]')
    rxn.Initialize()
    reacts1 = ['C[C@@H](F)CO', 'C[C@@H](F)CO |a:1|', 'C[C@@H](F)CO |o1:1|', 'C[C@@H](F)CO |&1:1|']
    reacts1 = [Chem.MolFromSmiles(x) for x in reacts1]

    prods = list(Enumerator.EnumerateReaction(rxn, (reacts1, ), uniqueProductsOnly=True))
    ps = Chem.SmilesWriteParams()
    print([Chem.MolToCXSmiles(tpl[0], ps, Chem.CXSmilesFields.CX_ENHANCEDSTEREO) for tpl in prods])
    self.assertEqual(len(prods), len(reacts1))


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
#  Copyright (C) 2005  greg Landrum and Rational Discovery LLC
#   All Rights Reserved
#
import gzip
import os
import unittest
from io import StringIO

from rdkit import RDConfig
from rdkit.DataStructs import ExplicitBitVect
from rdkit.DataStructs.VectCollection import VectCollection
from rdkit.ML import InfoTheory
from rdkit.ML.DecTree.BuildSigTree import BuildSigTree, _GenerateRandomEnsemble
from rdkit.ML.DecTree.SigTree import SigTreeNode
from rdkit.TestRunner import redirect_stdout


class TestCase(unittest.TestCase):

  def setUp(self):
    t1 = SigTreeNode(None, 'root', 0)

    t2 = SigTreeNode(t1, 'nodeL1', 1)
    t1.AddChildNode(t2)
    t3 = SigTreeNode(t2, 'nodeLTerm0', 0, isTerminal=1)
    t4 = SigTreeNode(t2, 'nodeLTerm1', 1, isTerminal=1)
    t2.AddChildNode(t3)
    t2.AddChildNode(t4)

    t2 = SigTreeNode(t1, 'nodeR1', 2)
    t1.AddChildNode(t2)
    t3 = SigTreeNode(t2, 'nodeRTerm0', 1, isTerminal=1)
    t4 = SigTreeNode(t2, 'nodeRTerm1', 0, isTerminal=1)
    t2.AddChildNode(t3)
    t2.AddChildNode(t4)
    self.tree = t1

  def test1(self):
    t1 = self.tree
    bv = ExplicitBitVect(5)

    ex = ['nm', bv]
    self.assertFalse(t1.ClassifyExample(ex))
    bv.SetBit(1)
    self.assertTrue(t1.ClassifyExample(ex))

    bv.SetBit(0)
    self.assertTrue(t1.ClassifyExample(ex))

    bv.SetBit(2)
    self.assertFalse(t1.ClassifyExample(ex))

  def test2(self):
    t1 = self.tree
    vc = VectCollection()

    bv = ExplicitBitVect(5)
    bv.SetBitsFromList([0])
    vc.AddVect(1, bv)

    bv = ExplicitBitVect(5)
    bv.SetBitsFromList([1, 2])
    vc.AddVect(2, bv)

    ex = ['nm', bv, 1]
    self.assertTrue(t1.ClassifyExample(ex))

    bv = ExplicitBitVect(5)
    bv.SetBitsFromList([0, 2])
    vc.AddVect(1, bv)
    ex = ['nm', bv, 1]
    self.assertFalse(t1.ClassifyExample(ex))

  def test3(self):
    examples = []

    bv = ExplicitBitVect(2)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['a', vc, 1])

    bv = ExplicitBitVect(2)
    bv.SetBit(1)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['c', vc, 0])

    bv = ExplicitBitVect(2)
    bv.SetBit(1)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['c2', vc, 0])

    bv = ExplicitBitVect(2)
    bv.SetBit(0)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['d', vc, 0])

    bv = ExplicitBitVect(2)
    bv.SetBit(0)
    vc = VectCollection()
    vc.AddVect(1, bv)
    bv = ExplicitBitVect(2)
    bv.SetBit(1)
    vc.AddVect(2, bv)
    examples.append(['d2', vc, 0])

    bv = ExplicitBitVect(2)
    bv.SetBit(0)
    bv.SetBit(1)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['d', vc, 1])

    bv = ExplicitBitVect(2)
    bv.SetBit(0)
    bv.SetBit(1)
    vc = VectCollection()
    vc.AddVect(1, bv)
    examples.append(['e', vc, 1])

    f = StringIO()
    with redirect_stdout(f):
      t = BuildSigTree(examples, 2, metric=InfoTheory.InfoType.ENTROPY, maxDepth=2, verbose=True)
    self.assertIn('Build', f.getvalue())

    self.assertEqual(t.GetName(), 'Bit-0')
    self.assertEqual(t.GetLabel(), 0)
    c0 = t.GetChildren()[0]
    self.assertEqual(c0.GetName(), 'Bit-1')
    self.assertEqual(c0.GetLabel(), 1)
    c1 = t.GetChildren()[1]
    self.assertEqual(c1.GetName(), 'Bit-1')
    self.assertEqual(c1.GetLabel(), 1)

    bv = ExplicitBitVect(2)
    bv.SetBit(0)
    vc = VectCollection()
    vc.AddVect(1, bv)
    bv = ExplicitBitVect(2)
    bv.SetBit(1)
    vc.AddVect(2, bv)
    r = t.ClassifyExample(['t', vc, 0])
    self.assertEqual(r, 0)

  def test4(self):
    import pickle
    gz = gzip.open(
      os.path.join(RDConfig.RDCodeDir, 'ML', 'DecTree', 'test_data', 'cdk2-few.pkl.gz'), 'rb')
    examples = pickle.load(gz, encoding='Latin1')
    t = BuildSigTree(examples, 2, maxDepth=3)
    self.assertEqual(t.GetLabel(), 2181)
    self.assertEqual(t.GetChildren()[0].GetLabel(), 2861)
    self.assertEqual(t.GetChildren()[1].GetLabel(), 8182)

  def test_GenerateRandomEnsemble(self):
    ensemble = _GenerateRandomEnsemble(2, 4)
    self.assertEqual(len(ensemble), 2)
    self.assertTrue(all(r < 4 for r in ensemble))

    ensemble = _GenerateRandomEnsemble(4, 4)
    self.assertEqual(len(ensemble), 4)
    self.assertTrue(all(r < 4 for r in ensemble))

    ensemble = _GenerateRandomEnsemble(4, 40)
    self.assertEqual(len(ensemble), 4)
    self.assertTrue(all(r < 40 for r in ensemble))


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# copyright 2000, greg landrum
"""

Here is the implementation for K-nearest neighbors

"""
from warnings import warn

warn('This module is deprecated and will be removed in the 2024.03 release', DeprecationWarning,
     stacklevel=2)



from Chem import rdmol
from Chem.rdmol import Atom, Bond, Mol


def ParseAtomBlock(lines, mol, nAtoms):
  for i in range(nAtoms):
    line = lines[i]
    pX = float(line[0:10])
    pY = float(line[10:20])
    pZ = float(line[20:30])
    symb = line[31:34].strip()
    newAt = rdmol.Atom(symb)
    newAt.setPos(pX, pY, pZ)

    chg = int(line[36:39])
    if chg in [1, 2, 3, 5, 6, 7]:
      newAt.setFormalCharge(4 - chg)

    # parse valence

    # parse rxn component

    mol.addAtom(newAt)


bondMap = {1: Bond.SINGLE, 2: Bond.DOUBLE, 3: Bond.TRIPLE, 4: Bond.AROMATIC}


def ParseBondBlock(lines, mol, nBonds):
  for i in range(nBonds):
    line = lines[i]
    id1 = int(line[0:3]) - 1
    id2 = int(line[3:6]) - 1
    order = int(line[6:9])
    order = bondMap.get(order, Bond.OTHER)
    b = Bond(order)
    b.setOwningMol(mol)
    b.setBeginAtomIdx(id1)
    b.setEndAtomIdx(id2)
    mol.addBond(b)


def ParseMolBlock(lines, mol):
  header = lines[0:3]
  counts = lines[3]
  nAtoms = int(counts[0:3])
  nBonds = int(counts[3:6])
  nLists = int(counts[6:9])
  chiralFlag = int(counts[12:15])
  nsText = int(counts[15:18])
  nRxnComponents = int(counts[18:21])
  nReactants = int(counts[21:24])
  nProducts = int(counts[24:27])
  nIntermediates = int(counts[27:30])

  ParseAtomBlock(lines[4:], mol, nAtoms)
  ParseBondBlock(lines[4 + nAtoms:], mol, nBonds)


if __name__ == '__main__':
  import sys
  fName = sys.argv[1]
  inF = open(fName, 'r')
  lines = inF.readlines()
  m = rdmol.Mol()
  ParseMolBlock(lines, m)

  print(m.getNumAtoms())
  m.debugMol()

  print(rdmol.MolToCDXML(m))



# $Id$
#
#  Copyright (C) 2006  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import unittest

from rdkit.Chem.FeatMaps import FeatMapParser, FeatMapUtils
from rdkit.Chem.FeatMaps.FeatMapUtils import MergeMetric
from rdkit.Geometry import Point3D


def feq(n1, n2, tol=1e-4):
  return abs(n1 - n2) <= tol


def pteq(p1, p2, tol=1e-4):
  return feq((p1 - p2).LengthSq(), 0.0, tol)


class TestCase(unittest.TestCase):

  def setUp(self):
    self.paramTxt = """
BeginParams
  family=Acceptor radius=0.5 profile=Box
EndParams
"""
    self.p = FeatMapParser.FeatMapParser()

  def test1Basics(self):
    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.1, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(3.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertFalse(FeatMapUtils.MergeFeatPoints(fm1))
    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 2)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.05, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(3.0, 0, 0)))

    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.1, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(3.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 2)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.05, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(3.5, 0, 0)))

    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.3, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.00, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.25, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=3.0
  family=Acceptor pos=(1.3, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance,
                                   mergeMethod=FeatMapUtils.MergeMethod.Average))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.00, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.25, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    self.p.SetData(txt)
    fm1 = self.p.Parse()
    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance,
                                   mergeMethod=FeatMapUtils.MergeMethod.WeightedAverage))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.00, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.225, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    self.p.SetData(txt)
    fm1 = self.p.Parse()
    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance,
                                   mergeMethod=FeatMapUtils.MergeMethod.UseLarger))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.00, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.2, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    # Order of points doesn't matter
    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.3, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=3.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()
    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance,
                                   mergeMethod=FeatMapUtils.MergeMethod.UseLarger))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.00, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.2, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    self.assertRaises(ValueError, FeatMapUtils.MergeFeatPoints, fm1,
                      FeatMapUtils.MergeMetric.Distance, mergeMethod='typo')

  def test1BasicsRepeated(self):
    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(0.7, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.3, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 5)
    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(0.7, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.0, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(1.25, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(3).GetPos(), Point3D(4.0, 0, 0)))

    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(0.7, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.125, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    self.assertTrue(FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Distance))
    self.assertTrue(fm1.GetNumFeatures() == 2)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(0.9125, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(4.0, 0, 0)))

  def test2ScoreBasics(self):
    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=3.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Overlap,
                                   mergeMethod=FeatMapUtils.MergeMethod.Average))
    self.assertTrue(fm1.GetNumFeatures() == 2)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.1, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(4.0, 0, 0)))

    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.1, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.3, 0.0, 0.0) weight=3.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Overlap,
                                   mergeMethod=FeatMapUtils.MergeMethod.Average))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.15, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.1, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

    txt = self.paramTxt + """
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.2, 0.0, 0.0) weight=1.0
  family=Acceptor pos=(1.6, 0.0, 0.0) weight=3.0
  family=Acceptor pos=(4.0, 0.0, 0.0) weight=1.0
EndPoints
    """
    self.p.SetData(txt)
    fm1 = self.p.Parse()

    self.assertTrue(fm1.GetNumFeatures() == 4)
    self.assertTrue(
      FeatMapUtils.MergeFeatPoints(fm1, FeatMapUtils.MergeMetric.Overlap,
                                   mergeMethod=FeatMapUtils.MergeMethod.Average))
    self.assertTrue(fm1.GetNumFeatures() == 3)
    self.assertTrue(pteq(fm1.GetFeature(0).GetPos(), Point3D(1.0, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(1).GetPos(), Point3D(1.4, 0, 0)))
    self.assertTrue(pteq(fm1.GetFeature(2).GetPos(), Point3D(4.0, 0, 0)))

  def test_CombineFeatMaps(self):
    data1 = self.paramTxt + "BeginPoints\nfamily=Acceptor pos=(1.0, 0.0, 0.0) weight=1.0\nEndPoints"
    data2 = self.paramTxt + "BeginPoints\nfamily=Acceptor pos=(0.0, 0.0, 0.0) weight=1.0\nEndPoints"

    self.p.SetData(data1)
    fm1 = self.p.Parse()
    self.assertEqual(fm1.GetNumFeatures(), 1)
    self.p.SetData(data2)
    fm2 = self.p.Parse()
    self.assertEqual(fm2.GetNumFeatures(), 1)

    # Combine without merging
    fm12 = FeatMapUtils.CombineFeatMaps(fm1, fm2)
    self.assertEqual(fm12.GetNumFeatures(), 2)

    # Combine with distance based merging
    fm12 = FeatMapUtils.CombineFeatMaps(fm1, fm2, mergeMetric=MergeMetric.Distance, mergeTol=1.0)
    self.assertEqual(fm12.GetNumFeatures(), 2)
    fm12 = FeatMapUtils.CombineFeatMaps(fm1, fm2, mergeMetric=MergeMetric.Distance, mergeTol=1.1)
    self.assertEqual(fm12.GetNumFeatures(), 1)

    self.assertRaises(ValueError, FeatMapUtils.CombineFeatMaps, fm1, fm2, mergeMetric='typo')


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (C) 2004-2006  greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import os
import unittest

from rdkit import Chem, Geometry, RDConfig
from rdkit.Chem import AllChem, ChemicalFeatures
from rdkit.Chem.Pharm3D import Pharmacophore


def feq(n1, n2, tol=1e-5):
  return abs(n1 - n2) <= tol


class TestCase(unittest.TestCase):

  def setUp(self):
    self.fdefBlock = """
                   DefineFeature HAcceptor1 [N,O;H0]
                      Family HBondAcceptor
                      Weights 1.0
                   EndFeature
                   DefineFeature HDonor1 [N,O;!H0]
                      Family HBondDonor
                      Weights 1.0
                   EndFeature
                   DefineFeature Aromatic1 c1ccccc1
                      Family Aromatic
                      Weights 1.0,1.0,1.0,1.0,1.0,1.0
                   EndFeature\n"""

    self.featFactory = ChemicalFeatures.BuildFeatureFactoryFromString(self.fdefBlock)
    self.feats = [
      ChemicalFeatures.FreeChemicalFeature('HBondAcceptor', 'HAcceptor1',
                                           Geometry.Point3D(0.0, 0.0, 0.0)),
      ChemicalFeatures.FreeChemicalFeature('HBondDonor', 'HDonor1',
                                           Geometry.Point3D(2.65, 0.0, 0.0)),
      ChemicalFeatures.FreeChemicalFeature('Aromatic', 'Aromatic1',
                                           Geometry.Point3D(5.12, 0.908, 0.0)),
    ]
    self.pcophore = Pharmacophore.Pharmacophore(self.feats)

  def test1Basics(self):
    pcophore = self.pcophore
    self.assertTrue(len(pcophore.getFeatures()) == 3)
    self.assertTrue(pcophore.getFeature(0))
    self.assertTrue(pcophore.getFeature(1))
    self.assertTrue(pcophore.getFeature(2))
    self.assertRaises(IndexError, pcophore.getFeature, 3)
    # print()
    # print(str(pcophore))
    self.assertIn('Aromatic', str(pcophore))

  def test2BoundSetting(self):
    pcophore = self.pcophore

    pcophore.setUpperBound(0, 1, 3.0)
    self.assertTrue(feq(pcophore.getUpperBound(0, 1), 3.0))
    self.assertTrue(feq(pcophore.getUpperBound(1, 0), 3.0))
    pcophore.setUpperBound(1, 0, 5.0)
    self.assertTrue(feq(pcophore.getUpperBound(0, 1), 5.0))
    self.assertTrue(feq(pcophore.getUpperBound(1, 0), 5.0))
    self.assertRaises(IndexError, pcophore.setUpperBound, 0, 3, 2.0)
    self.assertRaises(ValueError, pcophore.setUpperBound, 0, 3, 2.0, checkBounds=True)
    self.assertRaises(IndexError, pcophore.setUpperBound, 3, 0, 2.0)
    self.assertRaises(ValueError, pcophore.setUpperBound, 3, 0, 2.0, checkBounds=True)

    nfeatures = len(pcophore._feats)
    self.assertTrue(pcophore._checkBounds(0, 0))
    self.assertTrue(pcophore._checkBounds(0, nfeatures - 1))
    self.assertTrue(pcophore._checkBounds(nfeatures - 1, 0))
    self.assertTrue(pcophore._checkBounds(nfeatures - 1, nfeatures - 1))
    self.assertRaises(ValueError, pcophore._checkBounds, -1, 0)
    self.assertRaises(ValueError, pcophore._checkBounds, 0, -1)
    self.assertRaises(ValueError, pcophore._checkBounds, nfeatures, 0)
    self.assertRaises(ValueError, pcophore._checkBounds, 0, nfeatures)

    pcophore.setLowerBound(0, 1, 2.0)
    self.assertTrue(feq(pcophore.getLowerBound(0, 1), 2.0))
    self.assertTrue(feq(pcophore.getLowerBound(1, 0), 2.0))
    pcophore.setLowerBound(1, 0, 3.0)
    self.assertTrue(feq(pcophore.getLowerBound(0, 1), 3.0))
    self.assertTrue(feq(pcophore.getLowerBound(1, 0), 3.0))
    self.assertRaises(IndexError, pcophore.setLowerBound, 0, 3, 2.0)
    self.assertRaises(ValueError, pcophore.setLowerBound, 0, 3, 2.0, checkBounds=True)
    self.assertRaises(IndexError, pcophore.setLowerBound, 3, 0, 2.0)
    self.assertRaises(ValueError, pcophore.setLowerBound, 3, 0, 2.0, checkBounds=True)

  def test3Bound2DSetting(self):
    pcophore = self.pcophore

    pcophore.setUpperBound2D(0, 1, 3)
    self.assertTrue(pcophore.getUpperBound2D(0, 1) == 3)
    self.assertTrue(pcophore.getUpperBound2D(1, 0) == 3)
    pcophore.setUpperBound2D(1, 0, 5)
    self.assertTrue(pcophore.getUpperBound2D(0, 1) == 5)
    self.assertTrue(pcophore.getUpperBound2D(1, 0) == 5)
    self.assertRaises(IndexError, pcophore.setUpperBound2D, 0, 3, 2)
    self.assertRaises(ValueError, pcophore.setUpperBound2D, 0, 3, 2, checkBounds=True)
    self.assertRaises(IndexError, pcophore.setUpperBound2D, 3, 0, 2)
    self.assertRaises(ValueError, pcophore.setUpperBound2D, 3, 0, 2, checkBounds=True)

    pcophore.setLowerBound2D(0, 1, 3)
    self.assertTrue(pcophore.getLowerBound2D(0, 1) == 3)
    self.assertTrue(pcophore.getLowerBound2D(1, 0) == 3)
    pcophore.setLowerBound2D(1, 0, 5)
    self.assertTrue(pcophore.getLowerBound2D(0, 1) == 5)
    self.assertTrue(pcophore.getLowerBound2D(1, 0) == 5)
    self.assertRaises(IndexError, pcophore.setLowerBound2D, 0, 3, 2)
    self.assertRaises(ValueError, pcophore.setLowerBound2D, 0, 3, 2, checkBounds=True)
    self.assertRaises(IndexError, pcophore.setLowerBound2D, 3, 0, 2)
    self.assertRaises(ValueError, pcophore.setLowerBound2D, 3, 0, 2, checkBounds=True)

  def test4Github252(self):
    fdef = os.path.join(RDConfig.RDDataDir, 'BaseFeatures.fdef')
    feat_factory = ChemicalFeatures.BuildFeatureFactory(fdef)

    m1 = Chem.MolFromSmiles('Cc1ccccc1')
    feats = feat_factory.GetFeaturesForMol(m1)
    self.assertRaises(RuntimeError, lambda: Pharmacophore.Pharmacophore(feats))

    AllChem.Compute2DCoords(m1)
    Pharmacophore.Pharmacophore(feats)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
#  Copyright (C) 2004-2006  Greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
"""unit testing code for the AnalyzeComposite functionality

"""
import os
import pickle
import unittest

from rdkit import RDConfig
from rdkit.ML import AnalyzeComposite


class TestCase(unittest.TestCase):

  def setUp(self):
    self.baseDir = os.path.join(RDConfig.RDCodeDir, 'ML', 'test_data')

  def test1_Issue163(self):
    name1 = os.path.join(self.baseDir, 'humanoral.1.pkl')
    try:
      with open(name1, 'rb') as pklF:
        c1 = pickle.load(pklF)
    except Exception:
      c1 = None
    self.assertTrue(c1)
    name2 = os.path.join(self.baseDir, 'humanoral.2.pkl')
    try:
      with open(name2, 'rb') as pklF:
        c2 = pickle.load(pklF)
    except Exception:
      c2 = None
    self.assertTrue(c2)

    try:
      res = sorted(AnalyzeComposite.ProcessIt([c1, c2], verbose=-1))
    except Exception:
      import traceback
      traceback.print_exc()
      ok = 0
    else:
      ok = 1
    self.assertTrue(ok)

    self.assertEqual(res[0][0], 'BALABANJ')
    self.assertEqual(res[1][0], 'BERTZCT')
    self.assertEqual(res[-1][0], 'VSA_ESTATE9')
    for entry in res:
      self.assertEqual(len(entry), 5)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import os

from rdkit import Chem, RDConfig, rdBase
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors as rdMD

haveDescrs3D = hasattr(rdMD, 'CalcAUTOCORR3D')

import time
import unittest


def _gen3D(m, is3d, calculator):
  if not is3d:
    m = Chem.AddHs(m)
    ps = AllChem.ETKDG()
    ps.randomSeed = 0xf00d
    AllChem.EmbedMolecule(m, ps)
  return calculator(m)


class TestCase(unittest.TestCase):

  def setUp(self):
    self.dataDir = os.path.join(RDConfig.RDBaseDir, 'Code', 'GraphMol', 'Descriptors', 'test_data')
    self.suppl = Chem.SDMolSupplier(os.path.join(self.dataDir, 'PBF_egfr.sdf'), removeHs=False)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test1AUTOCORR2D(self):
    # not really a 3D descriptor, but this was added at the same time
    with open(os.path.join(self.dataDir, 'auto2D.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = rdMD.CalcAUTOCORR2D(m)
        for rv, nv in zip(split, vs):
          self.assertAlmostEqual(float(rv), nv, delta=0.05)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test2AUTOCORR3D(self):
    with open(os.path.join(self.dataDir, 'auto3D_dragon.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = _gen3D(m, True, rdMD.CalcAUTOCORR3D)
        for rv, nv in zip(split, vs):
          self.assertAlmostEqual(float(rv), nv, delta=0.05)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test3GETAWAY(self):
    with open(os.path.join(self.dataDir, 'GETAWAY.new.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = _gen3D(m, True, rdMD.CalcGETAWAY)
        for rv, nv in zip(split, vs):
          self.assertAlmostEqual(float(rv), nv, delta=0.05)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test4MORSE(self):
    with open(os.path.join(self.dataDir, 'MORSE.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = _gen3D(m, True, rdMD.CalcMORSE)
        for rv, nv in zip(split, vs):
          ref = float(rv)
          self.assertTrue(ref < 1 or abs(ref - nv) / ref < 0.02)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test5RDF(self):
    with open(os.path.join(self.dataDir, 'RDF.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = _gen3D(m, True, rdMD.CalcRDF)
        for rv, nv in zip(split, vs):
          ref = float(rv)
          self.assertTrue(ref < 0.5 or abs(ref - nv) / ref < 0.02)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def test6WHIM(self):
    with open(os.path.join(self.dataDir, 'whim.new.out')) as refFile:
      for i, m in enumerate(self.suppl):
        if i > 10:
          break
        nm = m.GetProp('_Name')
        inl = refFile.readline()
        split = inl.split('\t')
        self.assertEqual(split[0], nm)
        split.pop(0)
        vs = _gen3D(m, True, lambda x: rdMD.CalcWHIM(x, thresh=0.01))
        for rv, nv in zip(split, vs):
          self.assertAlmostEqual(float(rv), nv, delta=0.01)

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def testGithub2037(self):
    m = Chem.AddHs(Chem.MolFromSmiles("CCCCCCC"))
    cids = AllChem.EmbedMultipleConfs(m, 10)
    # start with defaults (which does not cache results):
    npr1s = []
    npr2s = []
    for cid in cids:
      npr1s.append(rdMD.CalcNPR1(m, confId=cid))
      npr2s.append(rdMD.CalcNPR2(m, confId=cid))
    for i in range(1, len(npr1s)):
      self.assertNotAlmostEqual(npr1s[0], npr1s[i])
      self.assertNotAlmostEqual(npr2s[0], npr2s[i])

    # now ensure that we can cache:
    npr1s = []
    npr2s = []
    for cid in cids:
      npr1s.append(rdMD.CalcNPR1(m, confId=cid, force=False))
      npr2s.append(rdMD.CalcNPR2(m, confId=cid, force=False))
    for i in range(1, len(npr1s)):
      self.assertAlmostEqual(npr1s[0], npr1s[i])
      self.assertAlmostEqual(npr2s[0], npr2s[i])

  @unittest.skipIf(not haveDescrs3D, "3d descriptors not present")
  def testGithub4167(self):
    with Chem.SDMolSupplier(os.path.join(self.dataDir, 'github4167.sdf'), removeHs=False,
                            sanitize=True) as suppl:
      m1 = suppl[0]
      m2 = suppl[1]
    m1.AddConformer(Chem.Conformer(m2.GetConformer()), assignId=True)
    v1_0 = rdMD.CalcSpherocityIndex(m1)
    v1_1 = rdMD.CalcSpherocityIndex(m1, confId=1, force=True)
    v2 = rdMD.CalcSpherocityIndex(m2)
    self.assertNotEqual(v1_0, v1_1)
    self.assertEqual(v1_1, v2)


if (__name__ == '__main__'):
  unittest.main()



# $Id$
#
#  Copyright (C) 2006  greg Landrum
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import unittest
from contextlib import closing
from io import StringIO

from rdkit.Chem.FeatMaps import FeatMapParser, FeatMaps


def feq(n1, n2, tol=1e-5):
  return abs(n1 - n2) <= tol


class TestCase(unittest.TestCase):
  data = """

ScoreMode=Best
DirScoreMode=DotFullRange

BeginParams
  family=Aromatic radius=2.5 width=1.0 profile=Triangle
  family=Acceptor radius=1.5
EndParams

# optional
BeginPoints
  family=Acceptor pos=(1.0, 0.0, 5.0) weight=1.25 dir=(1, 1, 0)
  family=Aromatic pos=(0.0,1.0,0.0) weight=2.0 dir=(0,0,1) dir=(0,0,-1)
  family=Acceptor pos=(1.0,1.0,2.0) weight=1.25
EndPoints

"""

  def test1Basics(self):
    p = FeatMapParser.FeatMapParser()
    p.SetData(self.data)
    fm = p.Parse()
    self.assertTrue(fm.scoreMode == FeatMaps.FeatMapScoreMode.Best)
    self.assertTrue(fm.dirScoreMode == FeatMaps.FeatDirScoreMode.DotFullRange)
    self.assertTrue(fm.GetNumFeatures() == 3)

    feats = fm.GetFeatures()
    self.assertTrue(feq(feats[0].weight, 1.25))
    self.assertTrue(feq(feats[1].weight, 2.0))
    self.assertTrue(feq(feats[2].weight, 1.25))

    self.assertTrue(len(feats[0].featDirs) == 1)
    self.assertTrue(len(feats[1].featDirs) == 2)
    self.assertTrue(len(feats[2].featDirs) == 0)

    fams = [x.GetFamily() for x in feats]
    self.assertTrue(fams == ['Acceptor', 'Aromatic', 'Acceptor'])

  def test_FeatMapParser(self):
    # We can use a string
    p = FeatMapParser.FeatMapParser(data=self.data)
    fm = p.Parse()
    self.assertEqual(fm.GetNumFeatures(), 3)
    self.assertEqual([x.GetFamily() for x in fm.GetFeatures()],
                     ['Acceptor', 'Aromatic', 'Acceptor'])

    # We can use a list of strings
    p = FeatMapParser.FeatMapParser(data=self.data.split('\n'))
    fm = p.Parse()
    self.assertEqual(fm.GetNumFeatures(), 3)
    self.assertEqual([x.GetFamily() for x in fm.GetFeatures()],
                     ['Acceptor', 'Aromatic', 'Acceptor'])

    # and a stream
    with closing(StringIO(self.data)) as file:
      p = FeatMapParser.FeatMapParser(file=file)
    fm = p.Parse()
    self.assertEqual(fm.GetNumFeatures(), 3)
    self.assertEqual([x.GetFamily() for x in fm.GetFeatures()],
                     ['Acceptor', 'Aromatic', 'Acceptor'])

  def test_ParseErrors(self):
    # Typos in scoreMode or dirscoreMode section
    data = "scoreMode = typo\nbeginParams\nfamily=Acceptor radius=1.5\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    data = "dirscoremode = typo\nbeginParams\nfamily=Acceptor radius=1.5\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    data = "typo = All\nbeginParams\nfamily=Acceptor radius=1.5\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    # Typos in paramBlock
    data = "beginTypo\nfamily=Acceptor radius=1.5\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    data = "beginParams\nfamily=Acceptor radius=1.5\nEndTypo"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    data = "beginParams\ntypo=Acceptor radius=1.5\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    data = "beginParams\nprofile=Typo\nEndParams"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)

    # Typos in points block
    data = "BeginPoints\npos=(1.0, 0.0, 5.0, 4.0)\nEndPoints"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(ValueError, p.Parse)

    data = "BeginPoints\npos=(1.0, 0.0, 5.0) typo=Acceptor\nEndPoints"
    p = FeatMapParser.FeatMapParser(data=data)
    self.assertRaises(FeatMapParser.FeatMapParseError, p.Parse)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
# Copyright (C) 2007,2008 Greg Landrum
#
#  @@ All Rights Reserved @@
#
import io
import os
import pickle
import random
import sys
import unittest

from rdkit import DataStructs as ds
from rdkit import RDConfig


def feq(v1, v2, tol=1e-4):
  return abs(v1 - v2) < tol


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def test1Int(self):
    """

    """
    v1 = ds.IntSparseIntVect(5)
    self.assertRaises(IndexError, lambda: v1[5])
    v1[0] = 1
    v1[2] = 2
    v1[3] = 3
    self.assertTrue(v1 == v1)
    self.assertTrue(v1.GetLength() == 5)

    v2 = ds.IntSparseIntVect(5)
    self.assertTrue(v1 != v2)
    v2 |= v1
    self.assertTrue(v2 == v1)

    v3 = v2 | v1
    self.assertTrue(v3 == v1)

    onVs = v1.GetNonzeroElements()
    self.assertTrue(onVs == {0: 1, 2: 2, 3: 3})

  def test2Long(self):
    """

    """
    l = 1 << 42
    v1 = ds.LongSparseIntVect(l)
    self.assertRaises(IndexError, lambda: v1[l])
    v1[0] = 1
    v1[2] = 2
    v1[1 << 35] = 3
    self.assertTrue(v1 == v1)
    self.assertTrue(v1.GetLength() == l)

    v2 = ds.LongSparseIntVect(l)
    self.assertTrue(v1 != v2)
    v2 |= v1
    self.assertTrue(v2 == v1)

    v3 = v2 | v1
    self.assertTrue(v3 == v1)

    onVs = v1.GetNonzeroElements()
    self.assertTrue(onVs == {0: 1, 2: 2, 1 << 35: 3})

  def test3Pickle1(self):
    """

    """
    l = 1 << 42
    v1 = ds.LongSparseIntVect(l)
    self.assertRaises(IndexError, lambda: v1[l + 1])
    v1[0] = 1
    v1[2] = 2
    v1[1 << 35] = 3
    self.assertTrue(v1 == v1)

    v2 = pickle.loads(pickle.dumps(v1))
    self.assertTrue(v2 == v1)

    v3 = ds.LongSparseIntVect(v2.ToBinary())
    self.assertTrue(v2 == v3)
    self.assertTrue(v1 == v3)

    #pickle.dump(v1,file('lsiv.pkl','wb+'))
    with open(os.path.join(RDConfig.RDBaseDir, 'Code/DataStructs/Wrap/testData/lsiv.pkl'),
              'r') as tf:
      buf = tf.read().replace('\r\n', '\n').encode('utf-8')
      tf.close()
    with io.BytesIO(buf) as f:
      v3 = pickle.load(f)
      self.assertTrue(v3 == v1)

  def test3Pickle2(self):
    """

    """
    l = 1 << 21
    v1 = ds.IntSparseIntVect(l)
    self.assertRaises(IndexError, lambda: v1[l + 1])
    v1[0] = 1
    v1[2] = 2
    v1[1 << 12] = 3
    self.assertTrue(v1 == v1)

    v2 = pickle.loads(pickle.dumps(v1))
    self.assertTrue(v2 == v1)

    v3 = ds.IntSparseIntVect(v2.ToBinary())
    self.assertTrue(v2 == v3)
    self.assertTrue(v1 == v3)

    #pickle.dump(v1,file('isiv.pkl','wb+'))
    with open(os.path.join(RDConfig.RDBaseDir, 'Code/DataStructs/Wrap/testData/isiv.pkl'),
              'r') as tf:
      buf = tf.read().replace('\r\n', '\n').encode('utf-8')
      tf.close()
    with io.BytesIO(buf) as f:
      v3 = pickle.load(f)
      self.assertTrue(v3 == v1)

  def test4Update(self):
    """

    """
    v1 = ds.IntSparseIntVect(5)
    self.assertRaises(IndexError, lambda: v1[6])
    v1[0] = 1
    v1[2] = 2
    v1[3] = 3
    self.assertTrue(v1 == v1)

    v2 = ds.IntSparseIntVect(5)
    v2.UpdateFromSequence((0, 2, 3, 3, 2, 3))
    self.assertTrue(v1 == v2)

  def test5Dice(self):
    """

    """
    v1 = ds.IntSparseIntVect(5)
    v1[4] = 4
    v1[0] = 2
    v1[3] = 1
    self.assertTrue(feq(ds.DiceSimilarity(v1, v1), 1.0))

    v1 = ds.IntSparseIntVect(5)
    v1[0] = 2
    v1[2] = 1
    v1[3] = 4
    v1[4] = 6
    v2 = ds.IntSparseIntVect(5)
    v2[1] = 2
    v2[2] = 3
    v2[3] = 4
    v2[4] = 4
    self.assertTrue(feq(ds.DiceSimilarity(v1, v2), 18.0 / 26.))
    self.assertTrue(feq(ds.DiceSimilarity(v2, v1), 18.0 / 26.))

  def test6BulkDice(self):
    """

    """
    sz = 10
    nToSet = 5
    nVs = 6
    import random
    vs = []
    for i in range(nVs):
      v = ds.IntSparseIntVect(sz)
      for j in range(nToSet):
        v[random.randint(0, sz - 1)] = random.randint(1, 10)
      vs.append(v)

    baseDs = [ds.DiceSimilarity(vs[0], vs[x]) for x in range(1, nVs)]
    bulkDs = ds.BulkDiceSimilarity(vs[0], vs[1:])
    for i in range(len(baseDs)):
      self.assertTrue(feq(baseDs[i], bulkDs[i]))

  def test6BulkTversky(self):
    """

    """
    sz = 10
    nToSet = 5
    nVs = 6
    import random
    vs = []
    for i in range(nVs):
      v = ds.IntSparseIntVect(sz)
      for j in range(nToSet):
        v[random.randint(0, sz - 1)] = random.randint(1, 10)
      vs.append(v)

    baseDs = [ds.TverskySimilarity(vs[0], vs[x], .5, .5) for x in range(1, nVs)]
    bulkDs = ds.BulkTverskySimilarity(vs[0], vs[1:], 0.5, 0.5)
    diceDs = [ds.DiceSimilarity(vs[0], vs[x]) for x in range(1, nVs)]
    for i in range(len(baseDs)):
      self.assertTrue(feq(baseDs[i], bulkDs[i]))
      self.assertTrue(feq(baseDs[i], diceDs[i]))

    bulkDs = ds.BulkTverskySimilarity(vs[0], vs[1:], 1.0, 1.0)
    taniDs = [ds.TanimotoSimilarity(vs[0], vs[x]) for x in range(1, nVs)]
    for i in range(len(bulkDs)):
      self.assertTrue(feq(bulkDs[i], taniDs[i]))
    taniDs = ds.BulkTanimotoSimilarity(vs[0], vs[1:])
    for i in range(len(bulkDs)):
      self.assertTrue(feq(bulkDs[i], taniDs[i]))

  def test7ToList(self):
    l = [0] * 2048
    nbits = 2048
    bv = ds.IntSparseIntVect(nbits)
    for j in range(nbits):
      x = random.randrange(0, nbits)
      l[x] = x
      bv[x] = x

    l2 = list(bv)
    l3 = bv.ToList()
    self.assertEqual(l, l2)
    self.assertEqual(l, l3)


if __name__ == '__main__':
  unittest.main()



import sys

from rdkit import TestRunner

tests = [
  ("python", "Utils.py", {}),
  ("python", "Torsions.py", {}),
  ("python", "Pairs.py", {}),
  ("python", "UnitTestDescriptors.py", {}),
]

longTests = []

if __name__ == '__main__':
  failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
  sys.exit(len(failed))



#
#  Copyright (C) 2007 Greg Landrum
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
import os
import unittest

from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.Subshape import (SubshapeAligner, SubshapeBuilder,
                                 SubshapeObjects)

debug = False
if not debug:
  SubshapeAligner.logger.setLevel(SubshapeAligner.RDLogger.ERROR)


class TestCase(unittest.TestCase):

  def test1(self):
    # computeCanonicalTransform returns more approximate eigenvalues/eigencvectors
    # when built against the native RDKit PowerEigenSolver, so unit test results
    # differ slightly
    builtAgainstEigen3 = hasattr(AllChem, 'ComputePrincipalAxesAndMomentsFromGyrationMatrix')
    if builtAgainstEigen3:
      expectedSkelPts = 15
      expectedAlgs = [0, 5, 21, 0]
      prunedAlgs = [0, 4, 11, 0]
    else:
      expectedSkelPts = 16
      expectedAlgs = [0, 5, 28, 0]
      prunedAlgs = [0, 4, 12, 0]
    filename = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'test_data', '5ht3ligs.sdf')
    suppl = Chem.SDMolSupplier(filename)
    builder = SubshapeBuilder.SubshapeBuilder()
    builder.gridDims = (20., 20., 10)
    builder.gridSpacing = 0.5
    builder.winRad = 4.

    ms = []
    shapes = []
    for m in suppl:
      m = Chem.AddHs(m, addCoords=True)
      AllChem.CanonicalizeConformer(m.GetConformer())
      ms.append(m)
      shape = builder(m, terminalPtsOnly=True)
      shapes.append(shape)

    self.assertEqual(len(ms), 4)
    self.assertEqual(len(shapes), 4)
    self.assertEqual([len(x.skelPts) for x in shapes], [5, 5, 5, 5])

    refShape = builder.GenerateSubshapeShape(ms[0])

    self.assertEqual(len(refShape.skelPts), expectedSkelPts)

    aligner = SubshapeAligner.SubshapeAligner()
    aligner.shapeDistTol = .30

    algStore = []
    for i, s1 in enumerate(shapes):
      if not i or not s1:
        algStore.append([])
        continue
      m1 = ms[i]
      alignments = aligner.GetSubshapeAlignments(ms[0], refShape, m1, s1, builder)
      algStore.append(alignments)
    self.assertEqual([len(x) for x in algStore], expectedAlgs)

    algStore = []
    for i, s1 in enumerate(shapes):
      if not i or not s1:
        algStore.append([])
        continue
      m1 = ms[i]
      alignments = list(aligner(ms[0], refShape, m1, s1, builder))
      algStore.append(alignments)
    self.assertEqual([len(x) for x in algStore], expectedAlgs)

    pruned = []
    for i, mi in enumerate(ms):
      alignments = algStore[i]
      pruned.append(SubshapeAligner.ClusterAlignments(mi, alignments, builder, neighborTol=0.15))
    self.assertEqual([len(x) for x in pruned], prunedAlgs)


class TestSubshapeObjects(unittest.TestCase):

  def test_SubshapeShape(self):
    shape = SubshapeObjects.SubshapeShape()
    self.assertEqual(shape.shapes, [])


if __name__ == '__main__':
  unittest.main()



import os
import sys
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import ChemicalForceFields, rdtrajectory
from rdkit.Chem.rdtrajectory import (ReadAmberTrajectory, ReadGromosTrajectory,
                                     Snapshot, Trajectory)


def feq(v1, v2, tol=1.0e-4):
  return abs(v1 - v2) < tol


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def testSnapshot(self):
    s = Snapshot([])
    e = False
    try:
      s.GetPoint2D(12)
    except Exception:
      e = True
    self.assertTrue(e)
    s = Snapshot([0.0, 0.0, 0.0])
    e = False
    try:
      s.GetPoint2D(0)
    except Exception:
      e = True
    self.assertTrue(e)

  def testTrajectory2D(self):
    dim = 2
    np = 10
    ns = 5
    traj = Trajectory(dim, np)
    self.assertEqual(traj.Dimension(), dim)
    self.assertEqual(traj.NumPoints(), np)
    c = []
    for i in range(np * dim):
      c.append(float(i))
    for i in range(ns):
      traj.AddSnapshot(Snapshot(c, float(i)))
    self.assertEqual(len(traj), ns)
    e = False
    try:
      traj.GetSnapshot(ns)
    except Exception:
      e = True
    self.assertTrue(e)
    e = False
    try:
      traj.GetSnapshot(0).GetPoint2D(np)
    except Exception:
      e = True
    self.assertTrue(e)
    for i in range(np):
      self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint2D(i).x, float(i * dim))
      self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint2D(i).y, float(i * dim + 1))
      e = False
      try:
        self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint3D(i).z, 0.0)
      except Exception:
        e = True
      self.assertFalse(e)
    for i in range(ns):
      self.assertAlmostEqual(traj.GetSnapshot(i).GetEnergy(), float(i))
    traj.RemoveSnapshot(0)
    self.assertEqual(len(traj), ns - 1)
    for i in range(ns - 1):
      self.assertAlmostEqual(traj.GetSnapshot(i).GetEnergy(), float(i + 1))
    traj.InsertSnapshot(0, Snapshot(c, 999.0))
    self.assertEqual(len(traj), ns)
    copySnapshot = Snapshot(traj.GetSnapshot(0))
    traj.AddSnapshot(copySnapshot)
    self.assertEqual(len(traj), ns + 1)
    self.assertAlmostEqual(traj.GetSnapshot(0).GetEnergy(), 999.0)
    self.assertAlmostEqual(traj.GetSnapshot(1).GetEnergy(), 1.0)
    self.assertAlmostEqual(traj.GetSnapshot(len(traj) - 1).GetEnergy(), 999.0)
    traj2 = Trajectory(traj)
    self.assertEqual(len(traj), len(traj2))

  def testTrajectory3D(self):
    dim = 3
    np = 10
    ns = 5
    traj = Trajectory(dim, np)
    self.assertEqual(traj.Dimension(), dim)
    self.assertEqual(traj.NumPoints(), np)
    c = []
    for i in range(np * dim):
      c.append(float(i))
    for i in range(ns):
      traj.AddSnapshot(Snapshot(c, float(i)))
    self.assertEqual(len(traj), ns)
    e = False
    try:
      traj.GetSnapshot(ns)
    except Exception:
      e = True
    self.assertTrue(e)
    e = False
    try:
      traj.GetSnapshot(0).GetPoint2D(np)
    except Exception:
      e = True
    self.assertTrue(e)
    for i in range(np):
      self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint3D(i).x, float(i * dim))
      self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint3D(i).y, float(i * dim + 1))
      self.assertAlmostEqual(traj.GetSnapshot(0).GetPoint3D(i).z, float(i * dim + 2))
      if (not i):
        e = False
        try:
          traj.GetSnapshot(0).GetPoint2D(i)
        except Exception:
          e = True
        self.assertTrue(e)
    for i in range(ns):
      self.assertAlmostEqual(traj.GetSnapshot(i).GetEnergy(), float(i))
    traj.RemoveSnapshot(0)
    self.assertEqual(len(traj), ns - 1)
    for i in range(ns - 1):
      self.assertAlmostEqual(traj.GetSnapshot(i).GetEnergy(), float(i + 1))
    traj.InsertSnapshot(0, Snapshot(c, 999.0))
    self.assertEqual(len(traj), ns)
    copySnapshot = Snapshot(traj.GetSnapshot(0))
    traj.AddSnapshot(copySnapshot)
    self.assertEqual(len(traj), ns + 1)
    self.assertAlmostEqual(traj.GetSnapshot(0).GetEnergy(), 999.0)
    self.assertAlmostEqual(traj.GetSnapshot(1).GetEnergy(), 1.0)
    self.assertAlmostEqual(traj.GetSnapshot(len(traj) - 1).GetEnergy(), 999.0)
    traj2 = Trajectory(traj)
    self.assertEqual(len(traj), len(traj2))

  def testReadAmber(self):
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords_bad.trx')
    traj = Trajectory(2, 0)
    ok = False
    try:
      ReadAmberTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    traj = Trajectory(3, 3)
    ok = False
    try:
      ReadAmberTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords_bad2.trx')
    ok = False
    try:
      traj = Trajectory(3, 3)
      ReadAmberTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords.trx')
    traj = Trajectory(3, 3)
    ReadAmberTrajectory(fName, traj)
    self.assertEqual(len(traj), 1)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords2.trx')
    traj = Trajectory(3, 3)
    ReadAmberTrajectory(fName, traj)
    self.assertEqual(len(traj), 2)

  def testReadAmberPython(self):
    # reimplemented the Amber trajectory reader in Python
    # let's check we get the same data as the C++ reader
    # (test for building a trajectory out of Snapshots from Python)
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords2.trx')
    traj = Trajectory(3, 3)
    nCoords = traj.NumPoints() * 3
    nSnapshots = 0
    hnd = open(fName, 'r')
    line = hnd.readline()
    lineNum = 0
    c = []
    i = 0
    while (line):
      lineNum += 1
      if (lineNum > 1):
        tok = line.strip().split()
        j = 0
        while ((i < nCoords) and (j < len(tok))):
          c.append(float(tok[j]))
          j += 1
          i += 1
        if (i == nCoords):
          nSnapshots += 1
          traj.AddSnapshot(Snapshot(c))
          c = []
          i = 0
          line = ' '.join(tok[j:]) + ' '
        else:
          line = ''
      else:
        line = ''
      line += hnd.readline()
    hnd.close()
    self.assertEqual(i, 0)
    self.assertEqual(nSnapshots, 2)
    traj2 = Trajectory(3, 3)
    ReadAmberTrajectory(fName, traj2)
    self.assertEqual(len(traj), len(traj2))
    self.assertEqual(traj.NumPoints(), traj2.NumPoints())
    for snapshotNum in range(len(traj)):
      for pointNum in range(traj.NumPoints()):
        for i in range(3):
          self.assertAlmostEqual(
            traj.GetSnapshot(snapshotNum).GetPoint3D(pointNum)[i],
            traj2.GetSnapshot(snapshotNum).GetPoint3D(pointNum)[i])

  def testReadGromos(self):
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords_bad.trc')
    traj = Trajectory(2, 0)
    ok = False
    try:
      ReadGromosTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    traj = Trajectory(3, 3)
    ok = False
    try:
      ReadGromosTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords_bad2.trc')
    ok = False
    try:
      traj = Trajectory(3, 3)
      ReadGromosTrajectory(fName, traj)
    except Exception:
      ok = True
    self.assertTrue(ok)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords.trc')
    traj = Trajectory(3, 3)
    ReadGromosTrajectory(fName, traj)
    self.assertEqual(len(traj), 1)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords2.trc')
    traj = Trajectory(3, 3)
    ReadGromosTrajectory(fName, traj)
    self.assertEqual(len(traj), 2)

  def testAddConformersFromTrajectory(self):
    molBlock = \
      '\n' \
      '     RDKit          3D\n' \
      '\n' \
      ' 71 74  0  0  0  0  0  0  0  0999 V2000\n' \
      '    8.2543    3.1901   -0.3005 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    7.4558    1.9712    0.0938 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    7.3934    1.0441   -0.9483 O   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    6.6660   -0.0533   -0.4641 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    5.1928    0.2346   -0.4609 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    4.3713   -0.9410   -0.5770 N   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    3.1852   -1.0034   -1.2291 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.2914    0.1276   -1.6316 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.9308   -0.4468   -1.9908 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.1417   -0.7821   -0.7545 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -0.1848    0.3695    0.0456 N   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -1.5661    0.7686   -0.0745 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -2.4768   -0.0640    0.8206 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -3.8874    0.1143    0.3941 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -4.6333   -0.9984    0.0264 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -6.0127   -0.9516   -0.0400 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -6.7062    0.1599    0.3963 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.0408    0.4828   -0.1977 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -7.7914    1.1180   -1.5591 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.7622    1.4403    0.7265 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.8409   -0.7397   -0.4395 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.9121   -1.6637    0.4258 O   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -9.7414   -0.7636   -1.5059 O   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -5.9736    1.2357    0.8565 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -4.5843    1.2252    0.8530 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.6263    1.4884   -0.3942 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.0541    1.0258   -0.4230 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.9225   -2.3317   -1.2963 N   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    3.6061   -2.9745   -0.3180 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    3.3554   -4.1536    0.3735 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    3.7653   -4.2712    1.6948 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    4.8254   -3.4613    2.0796 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    5.1978   -2.3436    1.3419 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    4.5694   -2.0799    0.1305 C   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    9.3138    3.1372    0.0031 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    7.8117    4.0754    0.1798 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    8.2358    3.3535   -1.4074 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    6.4027    2.2146    0.3634 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    7.9270    1.5444    1.0040 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    7.0677   -0.2415    0.5615 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    6.9530   -0.9105   -1.1025 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    4.9578    0.7259    0.5137 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    4.9985    0.9430   -1.3033 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.7171    0.7264   -2.4494 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.3994    0.2339   -2.6810 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    1.1342   -1.4171   -2.5076 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -0.7632   -1.3370   -1.0391 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.7845   -1.4394   -0.1311 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.0125    0.1989    1.0673 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -1.6672    1.8215    0.2925 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -1.8705    0.7271   -1.1337 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -2.3045    0.3159    1.8590 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -2.1980   -1.1367    0.7635 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -4.1513   -1.9468   -0.2114 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -6.6138   -1.7460   -0.4718 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -7.0727    0.4399   -2.0858 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -7.3144    2.1076   -1.4482 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.7609    1.1720   -2.1135 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.3137    2.4504    0.5729 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -8.6170    1.0817    1.7580 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -9.8244    1.4444    0.4200 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -6.4629    2.0541    1.3719 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '   -4.0445    2.0563    1.3058 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.3329    1.8224   -1.3991 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    0.4920    2.3164    0.3160 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.2025    0.3766    0.4766 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.7945    1.8369   -0.3969 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    2.4404   -4.6964    0.1303 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    3.3157   -5.0055    2.3587 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    5.4272   -3.7654    2.9380 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '    5.5668   -1.5069    1.9380 H   0  0  0  0  0  0  0  0  0  0  0  0\n' \
      '  1  2  1  0\n' \
      '  2  3  1  0\n' \
      '  3  4  1  0\n' \
      '  4  5  1  0\n' \
      '  5  6  1  0\n' \
      '  6  7  1  0\n' \
      '  7  8  1  0\n' \
      '  8  9  1  0\n' \
      '  9 10  1  0\n' \
      ' 10 11  1  0\n' \
      ' 11 12  1  0\n' \
      ' 12 13  1  0\n' \
      ' 13 14  1  0\n' \
      ' 14 15  2  0\n' \
      ' 15 16  1  0\n' \
      ' 16 17  2  0\n' \
      ' 17 18  1  0\n' \
      ' 18 19  1  0\n' \
      ' 18 20  1  0\n' \
      ' 18 21  1  0\n' \
      ' 21 22  2  0\n' \
      ' 21 23  1  0\n' \
      ' 17 24  1  0\n' \
      ' 24 25  2  0\n' \
      ' 11 26  1  0\n' \
      ' 26 27  1  0\n' \
      '  7 28  2  0\n' \
      ' 28 29  1  0\n' \
      ' 29 30  2  0\n' \
      ' 30 31  1  0\n' \
      ' 31 32  2  0\n' \
      ' 32 33  1  0\n' \
      ' 33 34  2  0\n' \
      ' 34  6  1  0\n' \
      ' 27  8  1  0\n' \
      ' 34 29  1  0\n' \
      ' 25 14  1  0\n' \
      '  1 35  1  0\n' \
      '  1 36  1  0\n' \
      '  1 37  1  0\n' \
      '  2 38  1  0\n' \
      '  2 39  1  0\n' \
      '  4 40  1  0\n' \
      '  4 41  1  0\n' \
      '  5 42  1  0\n' \
      '  5 43  1  0\n' \
      '  8 44  1  0\n' \
      '  9 45  1  0\n' \
      '  9 46  1  0\n' \
      ' 10 47  1  0\n' \
      ' 10 48  1  0\n' \
      ' 11 49  1  0\n' \
      ' 12 50  1  0\n' \
      ' 12 51  1  0\n' \
      ' 13 52  1  0\n' \
      ' 13 53  1  0\n' \
      ' 15 54  1  0\n' \
      ' 16 55  1  0\n' \
      ' 19 56  1  0\n' \
      ' 19 57  1  0\n' \
      ' 19 58  1  0\n' \
      ' 20 59  1  0\n' \
      ' 20 60  1  0\n' \
      ' 20 61  1  0\n' \
      ' 24 62  1  0\n' \
      ' 25 63  1  0\n' \
      ' 26 64  1  0\n' \
      ' 26 65  1  0\n' \
      ' 27 66  1  0\n' \
      ' 27 67  1  0\n' \
      ' 30 68  1  0\n' \
      ' 31 69  1  0\n' \
      ' 32 70  1  0\n' \
      ' 33 71  1  0\n' \
      'M  CHG  2  11   1  23  -1\n' \
      'M  END\n'
    mol = Chem.MolFromMolBlock(molBlock, removeHs=False)
    everySteps = 10
    maxIts = 1000
    gradTol = 0.01
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'Wrap', 'test_data',
                         'bilastine_trajectory.sdf')
    w = Chem.SDWriter(fName)
    field = ChemicalForceFields.MMFFGetMoleculeForceField(
      mol, ChemicalForceFields.MMFFGetMoleculeProperties(mol))
    (res, sv) = field.MinimizeTrajectory(everySteps, maxIts, gradTol)
    self.assertEqual(res, 0)
    traj = Trajectory(3, mol.GetNumAtoms(), sv)
    mol.RemoveConformer(0)
    traj.AddConformersToMol(mol)
    for nConf in range(mol.GetNumConformers()):
      mol.SetProp('ENERGY', '{0:.4f}'.format(traj.GetSnapshot(nConf).GetEnergy()))
      w.write(mol, nConf)
    w.close()
    traj.Clear()
    n1 = mol.GetNumConformers()
    traj.AddConformersToMol(mol)
    n2 = mol.GetNumConformers()
    self.assertEqual(n1, n2)
    # GetSnapshot should raise exception after Clear()
    e = False
    try:
      traj.GetSnapshot(0)
    except Exception:
      e = True
    self.assertTrue(e)

  def testAddConformersFromAmberTrajectory(self):
    mol = Chem.MolFromSmiles('CCC')
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords.trx')
    traj = Trajectory(3, mol.GetNumAtoms())
    ReadAmberTrajectory(fName, traj)
    self.assertEqual(len(traj), 1)
    for i in range(2):
      traj.AddConformersToMol(mol)
      self.assertEqual(mol.GetNumConformers(), i + 1)
      self.assertEqual(mol.GetConformer(i).GetNumAtoms(), 3)
      self.assertAlmostEqual(mol.GetConformer(i).GetAtomPosition(0).x, 0.1941767)
      self.assertAlmostEqual(mol.GetConformer(i).GetAtomPosition(2).z, -0.4088006)
    mol.RemoveAllConformers()
    e = False
    try:
      traj.AddConformersToMol(mol, 1)
    except Exception:
      e = True
    self.assertTrue(e)
    self.assertEqual(mol.GetNumConformers(), 0)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords2.trx')
    traj = Trajectory(3, mol.GetNumAtoms())
    ReadAmberTrajectory(fName, traj)
    self.assertEqual(len(traj), 2)
    traj.AddConformersToMol(mol)
    self.assertEqual(mol.GetNumConformers(), 2)
    mol.RemoveAllConformers()
    traj.AddConformersToMol(mol, 0, 0)
    self.assertEqual(mol.GetNumConformers(), 1)
    traj.AddConformersToMol(mol, 1)
    self.assertEqual(mol.GetNumConformers(), 2)

  def testAddConformersFromGromosTrajectory(self):
    mol = Chem.MolFromSmiles('CCC')
    rdbase = os.environ['RDBASE']
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords.trc')
    traj = Trajectory(3, mol.GetNumAtoms())
    ReadGromosTrajectory(fName, traj)
    self.assertEqual(len(traj), 1)
    for i in range(2):
      traj.AddConformersToMol(mol)
      self.assertEqual(mol.GetNumConformers(), i + 1)
      self.assertEqual(mol.GetConformer(i).GetNumAtoms(), 3)
      self.assertAlmostEqual(mol.GetConformer(i).GetAtomPosition(0).x, 1.941767)
      self.assertAlmostEqual(mol.GetConformer(i).GetAtomPosition(2).z, -4.088006)
    mol.RemoveAllConformers()
    e = False
    try:
      traj.AddConformersToMol(mol, 1)
    except Exception:
      e = True
    self.assertTrue(e)
    self.assertEqual(mol.GetNumConformers(), 0)
    fName = os.path.join(rdbase, 'Code', 'GraphMol', 'test_data', 'water_coords2.trc')
    traj = Trajectory(3, mol.GetNumAtoms())
    ReadGromosTrajectory(fName, traj)
    self.assertEqual(len(traj), 2)
    traj.AddConformersToMol(mol)
    self.assertEqual(mol.GetNumConformers(), 2)
    mol.RemoveAllConformers()
    traj.AddConformersToMol(mol, 0, 0)
    self.assertEqual(mol.GetNumConformers(), 1)
    traj.AddConformersToMol(mol, 1)
    self.assertEqual(mol.GetNumConformers(), 2)


if __name__ == '__main__':
  print("Testing Trajectory wrapper")
  unittest.main()



# -*- coding: utf-8 -*-
"""
molvs.utils
~~~~~~~~~~~

This module contains miscellaneous utility functions.

:copyright: Copyright 2016 by Matt Swain.
:license: MIT, see LICENSE file for more details.
"""

import functools
from itertools import tee


def memoized_property(fget):
  """Decorator to create memoized properties."""
  attr_name = f'_{fget.__name__}'

  @functools.wraps(fget)
  def fget_memoized(self):
    if not hasattr(self, attr_name):
      setattr(self, attr_name, fget(self))
    return getattr(self, attr_name)

  return property(fget_memoized)


def pairwise(iterable):
  """Utility function to iterate in a pairwise fashion."""
  a, b = tee(iterable)
  next(b, None)
  return zip(a, b)



#
#  Copyright (C) 2000  greg Landrum
#
""" unit tests for the Neural network trainer implementation

   this basically works out **all** of the network code

"""

import unittest

from rdkit.ML.Neural.ActFuncs import Sigmoid, TanH
from rdkit.ML.Neural.NetNode import NetNode
from rdkit.ML.Neural.Network import Network


class TestCaseActFuncs(unittest.TestCase):

  def test_Sigmoid(self):
    f = Sigmoid()
    self.assertAlmostEqual(f(0), 0.5)
    self.assertAlmostEqual(f(0), f.Eval(0))
    self.assertAlmostEqual(f.Deriv(0), 0.25)
    self.assertAlmostEqual(f(1), 1.0 - f(-1))
    self.assertAlmostEqual(f(2), 1.0 - f(-2))
    self.assertAlmostEqual(f.Deriv(1), f.Deriv(-1))
    self.assertAlmostEqual(f.Deriv(2), f.Deriv(-2))
    self.assertLess(f(1), f(2))
    self.assertLess(f.Deriv(2), f.Deriv(1))
    self.assertAlmostEqual(f.Deriv(1), f.DerivFromVal(f(1)))

  def test_TanH(self):
    f = TanH()
    self.assertAlmostEqual(f(0), 0.0)
    self.assertAlmostEqual(f(0), f.Eval(0))
    self.assertAlmostEqual(f.Deriv(0), 1.0)
    self.assertAlmostEqual(f(1), -f(-1))
    self.assertAlmostEqual(f(2), -f(-2))
    self.assertAlmostEqual(f.Deriv(1), f.Deriv(-1))
    self.assertAlmostEqual(f.Deriv(2), f.Deriv(-2))
    self.assertLess(f(1), f(2))
    self.assertLess(f.Deriv(2), f.Deriv(1))
    self.assertAlmostEqual(f.Deriv(1), f.DerivFromVal(f(1)))


class TestCaseNetNode(unittest.TestCase):

  def test_NetNode(self):
    # A node without input always returns 1
    nodeList = [None] * 2
    node = NetNode(0, nodeList)
    nodeList[0] = node
    valVect = [None] * 2
    self.assertEqual(node.Eval(valVect), 1)
    self.assertEqual(valVect, [1, None])

    node = NetNode(1, nodeList, inputNodes=[0], weights=[0.1])
    self.assertRaises(AssertionError, node.SetWeights, [0, 1])
    self.assertRaises(AssertionError, node.SetInputs, [0, 1])


class TestCaseNetwork(unittest.TestCase):

  def test_Network(self):
    nodeCounts = [2, 2, 1, 2]
    net = Network(nodeCounts)
    self.assertEqual(net.GetNumNodes(), 7)
    self.assertEqual(len(net.GetAllNodes()), 7)
    self.assertEqual(net.GetInputNodeList(), [0, 1])
    self.assertEqual(net.GetHiddenLayerNodeList(0), [2, 3])
    self.assertEqual(net.GetHiddenLayerNodeList(1), [4])
    self.assertEqual(net.GetOutputNodeList(), [5, 6])

    # We get a representation of the network
    s = str(net)
    self.assertIn('Network', s)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()
